"""
	Licensed Materials - Property of IBM Corp. 
IBM Rational Automation Framework 
(c) Copyright IBM Corporation 2003, 2012. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 

	
	File name: updateApp.py
	
	Script which updates files or modules in an Enterprise App deployed to WAS
"""


import sys
import os
from JavaConversionHelper import JavaConversionHelper
from ConfigWriter import ConfigWriterException
from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager

"""
This options filter is used in the WasDeploy class to 
filter out options specific to update.
"""
class _UpdateOptionsFilter:

	"""
	moduleName is the name of the module being updated.
	
	EX: PlantsByWebSphere.war
	
	"""
	def __init__(self, moduleName):
		self.moduleName = moduleName
	#endDef
	
	"""
		Includes all single value options 
		Excludes all multi value options that specify a different 
		module than the one being worked on
		
		name is the full property name
		value is the property value
		wasOption is the WAS specific name of the option
		
		Return 1(true) to include the option
		Return 0(false) to exclude the option
	"""
	def includeOption(self, name, value, wasOption):
		## handle special cases first
		if (wasOption == "MapRolesToUsers"):
			return 1
		if (wasOption == "MapRunAsRolesToUsers"):
			return 1
		
		if (name.find("options.multi.") > -1):
			## this is a module specific option, see if it pertains to our module
			if (value.find(self.moduleName) > -1):
				return 1
			else:
				return 0
		else:
			## this is a regular option, include it
			return 1
		#endIf
	#endDef
	
#endClass

"""
Updates an Application File
"""
def updateApp ( appName, updateprops, updatepropfile, keyPrefix):

	# read available installable apps
	global AdminApp
	global AdminConfig
	if (updateprops.has_key( keyPrefix + "FILE_UPDATE_CONTENTS")):
		contentsitem =  updateprops[keyPrefix + "FILE_UPDATE_CONTENTS"]	
	#endIf
		
	#endIf 
	if (updateprops.has_key( keyPrefix + "FILE_UPDATE_CONTENT_URI") and updateprops[keyPrefix + "FILE_UPDATE_CONTENT_URI"] != ""):
		contenturiitem = updateprops[keyPrefix + "FILE_UPDATE_CONTENT_URI"]		
	else:
		msg =  str(keyPrefix) + "FILE_UPDATE_CONTENT_URI is not specified in " + str(updatepropfile)
		LOGGER.error(msg)
		raise ConfigWriterException(msg)
	#endIf 
	
	if os.path.exists(contentsitem):
		appExists = WasDeploy.checkIfAppExists(appName )
		if (appExists):
			# update the app
			#print "Updating "+contentsitem+" in application "+appName+" with new file "+contenturiitem+"..."
			LOGGER.log("CRWWA5055I",[contentsitem,appName,contenturiitem])
			args = ['-operation', 'addupdate', '-contents', contentsitem, '-contenturi', contenturiitem]
			AdminApp.update(appName, 'file', args)
		else:
			# install the apps
			msg = "The application " + str(appName) + " isn't installed to update"
			LOGGER.error(msg)
			raise ConfigWriterException(msg)
		#endElse 
	else:
		msg = str(contentsitem) + " doesn't exist on the file system"
		LOGGER.error(msg)
		raise ConfigWriterException(msg)
	#endIf
#endDef 

"""
Updates a module within an Enterprise Application

Uses the appName.properties to identify any extra options which apply to this module.
"""
def updateAppModule ( appName, updateprops, updatepropfile, keyPrefix, cell, cluster):

	# read available installable apps
	global AdminApp
	global AdminConfig
	
	if (updateprops.has_key( keyPrefix + "MODULE_FILE_UPDATE_CONTENTS") and updateprops[keyPrefix + "MODULE_FILE_UPDATE_CONTENTS"] != ""):
		contentsitem =  updateprops[keyPrefix + "MODULE_FILE_UPDATE_CONTENTS"]
	else:
		msg = str(keyPrefix) + "MODULE_FILE_UPDATE_CONTENTS is not specified in " + str(updatepropfile)
		LOGGER.error(msg)
		raise ConfigWriterException(msg)
	#endIf 
	if (updateprops.has_key( keyPrefix + "MODULE_FILE_UPDATE_CONTENT_URI") and updateprops[keyPrefix + "MODULE_FILE_UPDATE_CONTENT_URI"] != ""):
		contenturiitem =  updateprops[keyPrefix + "MODULE_FILE_UPDATE_CONTENT_URI"]	
	else:
		msg = str(keyPrefix) + "MODULE_FILE_UPDATE_CONTENT_URI is not specified in " + str(updatepropfile)
		LOGGER.error(msg)
		raise ConfigWriterException(msg)
	#endIf 
	vh = ""
	modfilename = ""
	maptocluster = ""
	addmappingsvar = ""
	## leaving MODULE_FILE_UPDATE_VIRTUAL_HOST for backwards compatibility
	## deprecated in 7.1.1.3
	if (updateprops.has_key( keyPrefix + "MODULE_FILE_UPDATE_VIRTUAL_HOST")):
		vh =  updateprops[keyPrefix + "MODULE_FILE_UPDATE_VIRTUAL_HOST"]	
	#endIf
	## leaving MODULE_FILE_NAME for backwards compatibility
	## deprecated in 7.1.1.3
	if (updateprops.has_key(keyPrefix + "MODULE_FILE_NAME")):
		modfilename =  updateprops[keyPrefix + "MODULE_FILE_NAME"]
	#endIf 
	## leaving MAP_TO_CLUSTER for backwards compatibility
	## deprecated in 7.1.1.3
	if (updateprops.has_key(keyPrefix + "MAP_TO_CLUSTER")):
		maptocluster =  updateprops[keyPrefix + "MAP_TO_CLUSTER"]	
	#endIf
	## leaving ADDITIONAL_MAPPINGS for backwards compatibility
	## deprecated in 7.1.1.3
	if (updateprops.has_key( keyPrefix + "ADDITIONAL_MAPPINGS")):
		addmappingsvar =  updateprops[keyPrefix + "ADDITIONAL_MAPPINGS"]
	#endIf
	if os.path.exists(contentsitem):
		appExists = WasDeploy.checkIfAppExists(appName )
		if (appExists):
			maptoVHArgs = []
			mapmodtoservers = []
			# update the app
			#print "Updating module "+contentsitem+" in application "+appName
			LOGGER.log("CRWWA5056I",[contentsitem,appName])
			## deprecated in 7.1.1.3 - mapping modules to virtual hosts is done using an options string
			## in appName.properties
			if (vh != "" and modfilename != ""):
				maptoVHArgs =  ['-MapWebModToVH', [[modfilename, contenturiitem+',WEB-INF/web.xml', vh]]]
				#print "Updating module " + contentsitem + " and associating with virtual host " + vh
				LOGGER.log("CRWWA5057I",[contentsitem,vh])
			## deprecated in 7.1.1.3 - mapping modules to servers is done using an options string
			## in appName.properties
			if (maptocluster == "true" and modfilename != ""):
				#print "map to cluster is true"
				LOGGER.log("CRWWA5058I")
				serverstomap = 'WebSphere:cell='+cell+',cluster='+cluster
				if (addmappingsvar != ""):
					serverstomap = serverstomap+'+'+addmappingsvar
				#endIf
				mapmodtoservers =  ['-MapModulesToServers', [[modfilename, contenturiitem+',WEB-INF/web.xml', serverstomap ]]]
			#endIf
			args = ['-operation', 'addupdate', '-contents', contentsitem, '-contenturi', contenturiitem ]
			args.extend(maptoVHArgs)
			args.extend(mapmodtoservers)
			## 13605 - Enable this to allow users to specify split options in 
			##         appName.properties to apply while updating the module 
			## add the options specified in appName.properties specific to this module
			#optionsFilter = _UpdateOptionsFilter(contenturiitem)
			#optionsStr = WasDeploy.createOptionsFromDict(updateprops, "", optionsFilter)
			#optionsAsSequence = JavaConversionHelper.convertStringToSequence(optionsStr)
			#args.extend(optionsAsSequence)
			print AdminApp.update(appName, 'modulefile', args)
		else:
			# install the apps
			msg = "The application " + str(appName) + " isn't installed to update"
			LOGGER.error(msg)
			raise ConfigWriterException(msg)
		#endIf 
	else:
		msg = str(contentsitem) + " doesn't exist on the file system"
		LOGGER.error(msg)
		raise ConfigWriterException(msg)
	#endIf

#endDef 

LOGGER = _Logger("updateApp", MessageManager.RB_WEBSPHERE_WAS)
optDict, args = SystemUtils.getopt( sys.argv, 'appProps:;appFile:;scope:;scopename:;mode:;cellname:;appPropsOrig:' )

print "optDict =",optDict

# parse the properties into props
appPropsFile = optDict['appProps']
appPropOrigFile = optDict['appPropsOrig']
props = SystemUtils.getPropertyDict( appPropsFile )
if (props.has_key("UPDATES")):
	updates =  props["UPDATES"]		

	updatearray=updates.split(',')

	#print "Updating the following modules/files:", updatearray
	LOGGER.log("CRWWA5059I",[updatearray])

	for x in range(0,len(updatearray)): 
		
		## deprecated in 7.1.1.3 - all update properties are now stored in the same file
		## reducing the number of required files
		updatepropfile = appPropOrigFile.replace(".properties",("_"+updatearray[x]+".properties"))
		keyPrefix = ""
		if os.path.exists(updatepropfile):
			## backwards compatible behavior is to have a separate update file for each module
			#print "processing "+updatepropfile
			LOGGER.log("CRWWA5060I",[updatepropfile])
			updateprops = SystemUtils.getPropertyDict(updatepropfile)
		else:
			## new functionality combines the update properties into 1 file
			updateprops = props
			updatepropfile = appPropsFile
			keyPrefix = updatearray[x] + "."
		#endIf
		
		appName = ""
		if (updateprops.has_key( "APP_NAME") and updateprops["APP_NAME"] != ""):
			appName = updateprops["APP_NAME"]
			if (updateprops.has_key( keyPrefix + "FILE_UPDATE_CONTENTS") and updateprops[keyPrefix + "FILE_UPDATE_CONTENTS"] != ""):
				updateApp(appName, updateprops, updatepropfile, keyPrefix)
			else:
				#print "Info: " + keyPrefix + "FILE_UPDATE_CONTENTS is not specified in " + updatepropfile
				LOGGER.log("CRWWA5061I",[keyPrefix,updatepropfile])
				#print "      File contents will not be updated"
			#endIf
			if (updateprops.has_key(keyPrefix + "MODULE_FILE_UPDATE_CONTENTS") and updateprops[keyPrefix + "MODULE_FILE_UPDATE_CONTENTS"] != ""):
				updateAppModule(appName, updateprops, updatepropfile, keyPrefix, optDict['cellname'], optDict['scopename'])
			else:
				#print "Info: " + keyPrefix + "MODULE_FILE_UPDATE_CONTENTS is not specified in " + updatepropfile
				#print "      Modules will not be updated"
				LOGER.log("CRWWA5062I",[keyPrefix, updatepropfile])
			#endIf
		else:
			msg = ("APP_NAME is not defined in " + str(updatepropfile) +
			       " Modules and Files in the Application cannot be updated")
			LOGGER.error(msg)
			raise ConfigWriterException(msg)
		#endIf 
	#endFor
else:
	msg = ("'UPDATES' is not defined in " + str(appPropsFile) +
	      " Modules and Files in the Application cannot be updated")
	LOGGER.error(msg)
	raise ConfigWriterException(msg)
#endIf

AdminHelper.saveAndSyncCell()

