"""
	Licensed Materials - Property of IBM Corp. 
IBM Rational Automation Framework 
(c) Copyright IBM Corporation 2003, 2012. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 

	
	File name: FileHelper.py
	
	Contains utility methods for manipulating files
"""

import com
import java
import org

from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager
import com.ibm.rational.rafw.shared.util.PropertiesUtil as PropertiesUtil
import os
import re
import shutil
import socket
import time
from java.lang import *
from java.io import *
from java.util import *
import java.nio.charset.Charset as Charset
import java.lang.System as System
from com.ibm.rational.rafw.framework.util import FileUtil

# due to issue with some Java implementations, need to introspect on AbstractStringBuilder
try:
	import java.lang.AbstractStringBuilder as AbstractStringBuilder
	for n,f in AbstractStringBuilder.__dict__.items():
		x = org.python.core.PyReflectedFunction(n)
		for a in f.argslist:
			if a is None:
				continue
			m = StringBuffer.getMethod(n,a.args)
			x.addMethod(m)
		StringBuffer.__dict__[n] = x
	#endFor
except ImportError:
	# nothing to be done since this impl of Java doesn't use java.lang.AbstractStringBuilder
	pass
#endIf

newline = System.getProperty("line.separator")

"""
	This class provides helper methods for manipulating text
"""
class _FileHelper:
	
	# regular expression pattern to match end of line character => (\r\n)?
	_rexEOL = "(" + newline + ")?"
	
	def __init__(self):
		self.LOGGER = _Logger("FileHelper", MessageManager.RB_WEBSPHERE_WAS)
	#endDef

	"""
		Description: Checks if file is writable
		Returns True/False
	"""
	def isWritable(self, path):
		if not os.path.exists(path):
			raise "Path does not exist: " + path
		return File(path).canWrite()
	#endDef
	
	"""
		Description: Checks if file is readable
		Returns True/False
	"""
	def isReadable(self, path):
		if not os.path.exists(path):
			raise "Path does not exist: " + path
		return File(path).canRead()
	#endDef
	
	"""
		Description: Checks if file is executeable
		Returns True/False
	"""
	def isExecutable(self, path):
		if not os.path.exists(path):
			raise "Path does not exist: " + path
		return File(path).canExecute()
	#endDef
	
	"""
		Description: Private function used in FileHelper.find
	"""
	def __find__(self, searchPath, pattern, recursive, resultList=[]):
		filenames = os.listdir(searchPath)
		for filename in filenames:
			filepath = os.path.join(searchPath, filename)
			if re.search("^" + pattern + "$", filename) != None:
				resultList.append(filepath)
			if recursive and os.path.isdir(filepath) and not os.path.islink(filepath):
				self.__find__(filepath, pattern, recursive, resultList)
		return resultList
	#endDef
	
	"""
		Description: Finds files/folders matching given pattern. Returns full paths.
		Note: pattern is a regular expression
			example: use ".*" instead of "*"
	"""
	def find(self, searchPath, pattern, recursive=1):
		return self.__find__(searchPath, pattern, recursive, [])
	#endDef
	
	"""
		Description: Removes files and folders.
		os.remove() and os.rmdir() does not support IGNORE mode
		shutil.rmtree has a bug - IGNORE mode doesn't work, checked the source
	"""
	def rm(self, path, recursive=1, ignoreErrors=1):
		if ignoreErrors and not os.path.exists(path):
			return
		if os.path.isfile(path) or os.path.islink(path):
			os.remove(path)
		elif os.path.isdir(path) and recursive:
			shutil.rmtree(path)
		elif os.path.isdir(path):
			os.rmdir(path)
	#endDef
	
	"""
		Description: Write to file
		
		@param data: Data to be written to the file
		@param filePath: Full path to the file
		@param append: True if the data should be appended to the file, defaults to false
		@param charsetName: The name of the charset to be used when writing the file, 
		                    None if the default charset should be used.
		@raise IllegalArgumentException if the charset is specified and is not valid
		@deprecated:  Use FileUtil.writeFile instead
	"""
	def writeFile(self, data, filePath, append=0, charsetName=None):
		## now just passes along to Java version
		FileUtil.writeFile(data, filePath, append, charsetName)
	#endDef
		
	"""
		Description: Backups file
	"""
	def backupFile(self, filePath):
		strTime = time.strftime("%y.%m.%d_%H.%M.%S", time.gmtime())
		backupPath = filePath + "_" + strTime + ".bak"
		shutil.copy(filePath, backupPath)
		self.LOGGER.info("Backed up file: " + backupPath)
	#endDef
	
	"""
		Description: Returns file content as a java.lang.String
	"""
	"""
		Description: Updates a property file given a key and value
		Returns true if successful, otherwise false
	"""
	def updatePropertyFile(self, propertiesFile, key, value, backup = 1, overwrite = 1):
		valueMap = {};
		valueMap[key] = value;
		if propertiesFile is None or key is None or value is None:
			print "updatePropertyFile requires three arguments: propertiesFile, key, value"
			return 0
		if not self.isWritable(propertiesFile):
			print "File is not writable: " + propertiesFile
			return 0
		return self.updatePropertiesFile(propertiesFile, valueMap, backup, overwrite)
	#endDef
	
	"""
		Description: Updates a property file given a dictionary of key/value pairs
		Returns true if successful, otherwise false
	"""
	def updatePropertiesFile(self, propertiesFile, valueMap, backup = 1, overwrite = 1):
		if propertiesFile is None or valueMap is None:
			print "updatePropertyFile requires two arguments: propertiesFile, valueMap"
			return 0
		if not self.isWritable(propertiesFile):
			print "File is not writable: " + propertiesFile
			return 0
		
		# ant call may not setup these properties, so they come with value of ${BACKUP}, or ${OVERWRITE}
		if overwrite in (0, '0'):
			overwrite = 0
		else:
			overwrite = 1
			
		if backup in (0, '0'):
			backup = 0
		else:
			backup = 1
		
		properties = PropertiesUtil()
	
		try:
			properties.load(propertiesFile)
		except Exception, e:
			self.LOGGER.error("Error reading file " + propertiesFile + ": " + str(e))
			return 0
		
		hashKeys = valueMap.keys()
		hashKeys.sort()
		for key in hashKeys:
			value = str(valueMap[key])
			currentValue = properties.getProperty(key)

			if currentValue is None:
				self.LOGGER.debug("Adding key " + key + " to properties file.")
			elif overwrite:
				self.LOGGER.debug("Updating key " + key + " in properties file. Old key value: " + currentValue)
			else:
				self.LOGGER.info("Property is already set, overwriting not permitted. Key: " + key + ", Value: " + currentValue)
				return 1
			properties.setProperty(key, value)
		#endFor
		
		try:
			if (backup):
				self.backupFile(propertiesFile)
			properties.store(propertiesFile)
		except Exception, e:
			self.LOGGER.error("Error writing file " + propertiesFile + ": " + str(e))
			return 0
		
		return 1
	#endDef
	
	"""
		Description: Runs an external diff tool
		Arguments:
			compareFilesInput - temporary file containing pairs file_1!=file_2
			externalDiffTool - this argument is value of EXTERNAL_DIFF property in configure.properties
			                   example value: EXTERNAL_DIFF=diff ${FILE_1} ${FILE_2}
			relativeHome   -- path on the framework server that the files in compareFilesInput are relative to
			
		@deprecated: Replaced by ExecuteAction java class
	"""
	def compareFiles(self, compareFilesInput, externalDiffTool, relativeHome):
		file = File(compareFilesInput)
		
		try:
			fileReader = FileReader(file) 
			bufferedReader = BufferedReader(fileReader)
			
			# read file line by line			
			while 1:
				s = bufferedReader.readLine()
				if s is None:
					break
				
				file1, file2 = s.split("!=")
				
				# example: EXTERNAL_DIFF=diff ${FILE_1} ${FILE_2}
				externalDiffToolTemp = externalDiffTool
				absFile1 = File(relativeHome + File.separator + file1) 
				absFile2 = File(relativeHome + File.separator + file2) 
				externalDiffToolTemp = externalDiffToolTemp.replace("${FILE_1}", absFile1.getAbsolutePath())
				externalDiffToolTemp = externalDiffToolTemp.replace("${FILE_2}", absFile2.getAbsolutePath())
				
				print "Executing: " + externalDiffToolTemp
				print ""
				os.system(externalDiffToolTemp)
				print ""
			#endWhile
			
			bufferedReader.close()
		except Exception, e:
			raise "Error reading file " + compareFilesInput + ": " + str(e)
	#endDef
		
	"""
		Description: Update the add node and remove node command to use more memory
		Returns true if successful, otherwise false
	"""
	def updateAddRemoveNode(self, wasHome, replacementString):
		if wasHome is None:
			print "updateAddRemoveNode requires one argument: WAS_HOME"
			return 0
		
		addNodeScript = wasHome + "/bin/addNode" + self._getScriptExt()
		removeNodeScript = wasHome + "/bin/removeNode" + self._getScriptExt()
		
		if not self.isWritable(addNodeScript):
			print "File is not writable: " + addNodeScript
			return 0
		if not self.isWritable(removeNodeScript):
			print "File is not writable: " + removeNodeScript
			return 0

		#cat $addNodeScript | grep -v "^#" | sed 's/^.*\$WAS_DEBUG.*$/  \$WAS_DEBUG -Xms256m -Xmx1024m \\/' > $addNodeScript.temp
		addNodeScriptContent = String(FileUtil.readFileAsString(addNodeScript, None))
		pattern = r".*\$WAS_DEBUG.*" + _FileHelper._rexEOL
		replacement = r"  \$WAS_DEBUG " + replacementString + r" \\" + newline
		addNodeScriptContent = addNodeScriptContent.replaceAll(pattern, replacement)
		self.backupFile(addNodeScript)
		self.writeFile(addNodeScriptContent, addNodeScript)
		
		#cat $removeNodeScript | grep -v "^#" | sed 's/^.*\$WAS_DEBUG.*$/ \$WAS_DEBUG -Xms256m -Xmx1024m \\/' > $removeNodeScript.temp
		removeNodeScriptContent = String(FileUtil.readFileAsString(removeNodeScript, None))
		pattern = r".*\$WAS_DEBUG.*" + _FileHelper._rexEOL
		replacement = r"  \$WAS_DEBUG " + replacementString + r" \\" + newline
		removeNodeScriptContent = removeNodeScriptContent.replaceAll(pattern, replacement)
		self.backupFile(removeNodeScript)
		self.writeFile(removeNodeScriptContent, removeNodeScript)
		
		return 1
	#endDef
	
	"""
		Description: Update the startNode,startManager,startServer scripts
		Returns true if successful, otherwise false
	"""
	def addDB2ProfileToStartScripts(self, wasHome, db2Home):
		if wasHome is None or db2Home is None:
			print "addDB2ProfileToStartScripts requires two arguments: wasHome, db2Home"
			return 0
		
		setupCmdLineScript = wasHome + "/bin/setupCmdLine" + self._getScriptExt()
		
		if not self.isWritable(setupCmdLineScript):
			print "File is not writable: " + setupCmdLineScript
			return 0
		
		print "Updating " + setupCmdLineScript + " to source " + db2Home + "/sqllib/db2profile"
		#echo ". ${db2Home}/sqllib/db2profile" >> $setupCmdLineScript
		data = newline + ". " + db2Home + "/sqllib/db2profile"
		self.backupFile(setupCmdLineScript)
		# append this line to the end of file
		self.writeFile(data, setupCmdLineScript, 1)
		
		return 1
	#endDef
	
	"""
		Description: Updates Release File
		Returns true if successful, otherwise false
		@deprecated:  Use FileUtil.updateReleaseFile instead
	"""
	def updateReleaseFile(self, portalClusterHome, fileName = "release.xml"):
		try:
			releaseFile = portalClusterHome + "/" + fileName
			self.backupFile(releaseFile)
			
			FileUtil.updateReleaseFile(portalClusterHome, fileName)
			return 1
		except Exception, e:
			print str(e)
			return 0
	#endDef
	
	"""
		Description: Updates Virtual Portal Release File
		Returns true if successful, otherwise false
		@deprecated:  Use FileUtil.updateVPReleaseFile instead
	"""
	def updateVPReleaseFile(self, portalClusterHome, virtualPortal, fileName = "release.xml"):
		try:
			releaseFile = portalClusterHome + "/virtualportal/" + virtualPortal + "/" + fileName
			self.backupFile(releaseFile)
			
			FileUtil.updateVPReleaseFile(portalClusterHome, virtualPortal, fileName)
			return 1
		except Exception, e:
			print str(e)
			return 0
	#endDef
	
	# TODO should be able to define this in ExecutionEnvironment 
	# because it is being set by logic in that script also
	def _getScriptExt(self):
		OSTYPE_WINDOWS = "windows"
		
		osname = System.getProperty("os.name")

		# define OS specific variables
		if (osname.lower().startswith(OSTYPE_WINDOWS)):
			scriptExt = ".bat"
		else:
			scriptExt = ".sh"
		return scriptExt
	#endDef
#endClass

## define global singletons
FileHelper = _FileHelper()

