"""
	Licensed Materials - Property of IBM Corp. 
IBM Rational Automation Framework 
(c) Copyright IBM Corporation 2003, 2012. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 

	
	File name: ConfigMediator.py
	
	This contains methods and classes to facilitate writing, reading and comparing WebSphere configuration data.
"""

import sys

from ConfigComparor import *
from ConfigReader import *
from ConfigWriter import *
from ConfigFileWriter import *
from ConfigFileReader import *
from ConfigValidator import *
from com.ibm.rational.rafw.wsadmin.performance import Performance

import AdminConfig

"""
############## CLASS :: _ConfigMediator ##################
# This class provides helper methods for WAS             #
# Administration                                         #
#                                                        #
##########################################################
"""
class _ConfigMediator:

	"""
	##--------------------------------------------------------------------
	## Helper method which outlines a common algorithm for importing WAS configuration.
	## 
	## This method first reads all config data, then injects or replaces that data
	## into the supplied file. 
	## 
	## @param scope: The WAS scope name to import config from
	## @param scopeType: cell/node/server/cluster - Used to filter out objects of sub-scope
	## @param xmlFile: The file to inject data into
	## @param marker:  used to determine the container node for data injection
	## @param typeNames: Array of WAS config types to read
	##
	##--------------------------------------------------------------------
	"""
	def importConfig(self, scope, scopeType, dataFile, marker, typeNames, excludedTypes = [], myConfigReader = None):
		Performance.start("ConfigMediator.importConfig")
		if (myConfigReader is None):
			myConfigReader = ConfigReader()
		#endIf
		data = []
		for typeName in typeNames:
			data.extend(myConfigReader.readConfigData(scope, scopeType, typeName, excludedTypes))
		#endFor
		
		GenericConfigFileWriter.processBasicFile (dataFile, data, marker)
		Performance.end("ConfigMediator.importConfig")
	#endDef

	"""
	##--------------------------------------------------------------------
	## Helper method which outlines a common algorithm for importing WAS configuration.
	## 
	## This method first reads all config data, then injects or replaces that data
	## into the supplied file. 
	## 
	## @param scopeid: The WAS parentid to import config from
	## @param scopeType: cell/node/server/cluster - Used to filter out objects of sub-scope
	## @param xmlFile: The file to inject data into
	## @param marker:  used to determine the container node for data injection
	## @param typeNames: Array of WAS config types to read
	##
	##--------------------------------------------------------------------
	"""
	def importConfigUsingParentId(self, scopeId, scopeType, dataFile, marker, typeNames):
	
		Performance.start("ConfigMediator.importConfigUsingParentId")
		myConfigReader = ConfigReader()
		data = []
		for typeName in typeNames:
			data.extend(myConfigReader.readConfigDataUsingParentId(scopeId, scopeType, typeName))
		#endFor
		
		GenericConfigFileWriter.processBasicFile (dataFile, data, marker)
		Performance.end("ConfigMediator.importConfigUsingParentId")
	#endDef

	"""
	##--------------------------------------------------------------------
	## Helper method which outlines a common algorithm for creating WAS configuration.
	##
	## This method removes all existing WAS configuration for the given types 
	## under the given scope
	## 
	## @param scope: The scope to create configuration under
	## @param scopeType: cell/node/cluster/server - Used to filter out incorrectly scoped objects
	## @param xmlFile: The xml file name to read configuration data from
	## @param marker:  used to determine the container node to read data within
	## @param typeNames: Array of WAS configuration types which will be read 
	##                   from the xml File and created
	##
	##--------------------------------------------------------------------
	"""
	def createConfig(self, scope, scopeType, xmlFile, marker, typeNames, excludedNodeNames = []):
		Performance.start("ConfigMediator.createConfig")
		scopeid = AdminConfig.getid(scope)
		if (len(scopeid) == 0):
			raise ("ERROR: unable to find parent scope: " + scope +  ".  Cannot write WAS config.")
		#endIf
		self.createConfigUsingParentId(scopeid, scopeType, xmlFile, marker, typeNames, excludedNodeNames)
		Performance.end("ConfigMediator.createConfig")
	#endDef

	"""
	##--------------------------------------------------------------------
	## Helper method which outlines a common algorithm for creating WAS configuration.
	##
	## This method removes all existing WAS configuration for the given types 
	## under the given scope
	## 
	## @param scopeid: The parent scopeid to create configuration under
	## @param scopeType: cell/node/cluster/server - Used to filter out incorrectly scoped objects
	## @param xmlFile: The xml file name to read configuration data from
	## @param marker:  used to determine the container node to read data within
	## @param typeNames: Array of WAS configuration types which will be read 
	##                   from the xml File and created
	##
	##--------------------------------------------------------------------
	"""
	def createConfigUsingParentId(self, scopeid, scopeType, xmlFile, marker, typeNames, excludedNodeNames = []):
		Performance.start("ConfigMediator.createConfigUsingParentId")
		xmlProp = ConfigFileReader.openXmlConfig(xmlFile)
		
		## accomodate for different file formats
		xmlPropNode = xmlProp.findRAFWContainerNode(marker)
	
		if (len(scopeid) == 0):
			print "ERROR: unable to find parent scope: " + scopeid
			print "Cannot write WAS config. Returning without doing anything"
			return
		#endIf
		
		myConfigWriter = ConfigWriter();
		for typeName in typeNames:
			shouldUpdate = 0
			if (typeName != "ResourceEnvironmentProvider"):
				myConfigWriter.removeExistingConfig(scopeType, typeName, scopeid)
			else:
				shouldUpdate = 1
			#endIf
			nodeArray = xmlPropNode.getFilteredChildrenArray(typeName)
			for xmlNode in nodeArray:
				if shouldUpdate:
					myConfigWriter.trueUpdateWASObject([xmlNode], str(scopeid), typeName, excludedNodeNames)
				else:
					myConfigWriter.createWASObject(xmlNode, scopeid, excludedNodeNames)
			#endFor
		#endFor
		Performance.end("ConfigMediator.createConfigUsingParentId")
	#endDef
	
	def augmentConfigUsingParentId(self, scopeid, scopeType, xmlFile, marker, typeNames, excludedNodeNames = [], configValidator = None):
		Performance.start("ConfigMediator.augmentConfigUsingParentId")
		if (configValidator is None):
			configValidator = ConfigValidator()
		#endIf
		
		xmlProp = ConfigFileReader.openXmlConfig(xmlFile)
		
		## accomodate for different file formats
		xmlPropNode = xmlProp.findRAFWContainerNode(marker)
	
		if (len(scopeid) == 0):
			print "ERROR: unable to find parent scope: " + scopeid
			print "Cannot write WAS config. Returning without doing anything"
			return
		#endIf
		
		myConfigWriter = ConfigWriter();
		for typeName in typeNames:
			if (typeName == "JDBCProvider"):
				skipReadOnlyAttribute = SkipNamedAttribute("providerType")
			else:
				skipReadOnlyAttribute = None
			nodeArray = xmlPropNode.getFilteredNodeArray(typeName)	
			for xmlNode in nodeArray:
				childId = configValidator.validate2(xmlNode, scopeid)
				uniqueAttrName = configValidator.getUniqueAttributeName()
				if (childId is None):
					print "Info: Creating " + typeName + " with " + uniqueAttrName + " " + xmlNode.getAttrValue(uniqueAttrName)
					myConfigWriter.createWASObject(xmlNode, scopeid, excludedNodeNames)
				else:
					if (SystemUtils.updateOnAugment()):
						xmlNode.addSkipAttributeOnWriteFilter(skipReadOnlyAttribute)
						myConfigWriter.modify(childId, xmlNode, excludedNodeNames)
					else:
						print "Warning: " + typeName + " with " + uniqueAttrName + " " + xmlNode.getAttrValue(uniqueAttrName, Globals.ATTRIBUTE_VALUE_MISSING) + " will not be created."
				#endIf
			#endFor
		#endFor
		Performance.end("ConfigMediator.augmentConfigUsingParentId")
	#endDef
	
	def compareConfig(self, scope, scopeType, xmlFile, marker, typeNames, excludedTypes = [], myConfigReader = None):
		Performance.start("ConfigMediator.compareConfig")
		if (myConfigReader is None):
			myConfigReader = ConfigReader()
		#endIf
		xmlProp = ConfigFileReader.openXmlConfig(xmlFile)
		
		## accomodate for different file formats
		xmlPropNode = xmlProp.findRAFWContainerNode(marker)
		
		xmlConfigReader = XMLConfigReader()
		dataDiffers = 0
		# get wasConfig
		for typeName in typeNames:
			wasConfig = myConfigReader.readConfigData(scope, scopeType, typeName, excludedTypes)

			# get rafwConfig
			filteredNodes = xmlPropNode.getFilteredNodeArray(typeName)
			rafwConfig = xmlConfigReader.readXmlConfig(filteredNodes)
			thisDataDiffers = not ConfigComparor.compare(typeName, wasConfig, rafwConfig)
			dataDiffers = (dataDiffers or thisDataDiffers)
		#endFor
		if (dataDiffers):
			# produce a file containing the current configuration so that user 
			# can compare using their favorite tool
			# copy the file
			pass
			# write the contents to the copy
			#GenericConfigFileWriter.processBasicFile (xmlFileCopy, data, marker)
		#endIf
		Performance.end("ConfigMediator.compareConfig")
	#endDef
	
	def compareConfigUsingParentId(self, scopeId, scopeType, xmlFile, marker, typeNames, excludedTypes = [], myConfigReader = None):
		Performance.start("ConfigMediator.compareConfigUsingParentId")
		if (myConfigReader is None):
			myConfigReader = ConfigReader()
		#endIf
		xmlProp = ConfigFileReader.openXmlConfig(xmlFile)
		xmlConfigReader = XMLConfigReader()
		
		## accomodate for different file formats
		xmlPropNode = xmlProp.findRAFWContainerNode(marker)
		
		# get wasConfig
		for typeName in typeNames:
			wasConfig = myConfigReader.readConfigDataUsingParentId(scopeId, scopeType, typeName, excludedTypes)

			# get rafwConfig
			filteredNodes = xmlPropNode.getFilteredNodeArray(typeName)
			rafwConfig = xmlConfigReader.readXmlConfig(filteredNodes)
			ConfigComparor.compare(typeName, wasConfig, rafwConfig)
		#endFor
		Performance.end("ConfigMediator.compareConfigUsingParentId")
	#endDef

#endClass

## define global singleton
ConfigMediator = _ConfigMediator()	

# Constants for MODE
MODE_EXECUTE="execute"
MODE_IMPORT="import"
MODE_COMPARE="compare"
MODE_AUGMENT="augment"

