"""
    Licensed Materials - Property of IBM Corp. 
IBM Rational Automation Framework 
(c) Copyright IBM Corporation 2003, 2012. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 

	
	File name: ConfigReader.py
	
	This contains methods and classes to facilitate reading WebSphere configuration data.
"""

import sys
from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager
from AdminHelper import *
from ConfigReaderException import ConfigReaderException
from SystemUtils import *

import AdminConfig
import Globals
import traceback
import java
import com
import com.ibm.websphere.management.configservice.ConfigServiceFactory as ConfigServiceFactory

from com.ibm.rational.rafw.wsadmin.performance import Performance
from java.util import HashMap
from java.util import ArrayList
from com.ibm.rational.rafw.wsadmin.websphere.config import WsadminConfig

newline = java.lang.System.getProperty("line.separator")

############## CLASS :: ConfigElements ###################
# This class is used by ConfigReader to store data while #
# reading                                                #
#                                                        #
# THIS CLASS MAINTAINS STATE -- CANNOT BE A SINGLETON    #
#                                                        #
##########################################################
class ConfigElements:
	def __init__(self):
		self.highPriorityIds = {}
		self.lowPriorityIds = {}
		self.referenceIds = {}
	#endDef

	"""
	ReferenceIds is a Dict<String, Sequence>
	"""
	def getReferenceIds(self):
		return self.referenceIds
	#endDef
	
	def addReferenceIds(self, key, ids):
		if (not self.referenceIds.has_key(key)):
			self.referenceIds[key] = []
		#endIf
		self.referenceIds[key].extend(ids)
	#endDef
	
	"""
		Adds the supplied key=id pair to the referenceIds data
		structure
	"""
	def addReferenceId(self, key, id):
		self.addReferenceIds(key, [id])
	#endDef
	
	def addHighPriorityId(self, key, id):
		if (not self.highPriorityIds.has_key(key)):
			self.highPriorityIds[key] = []
		#endIf
		self.highPriorityIds[key].append(id)
	#endDef
	
	def addLowPriorityId(self, key, id):
		self.initializeLowPriorityId(key)
		self.lowPriorityIds[key].append(id)
	#endDef
	
	def initializeLowPriorityId(self, key):
		if (not self.lowPriorityIds.has_key(key)):
			self.lowPriorityIds[key] = []
		#endIf
	#endIf
	
#endClass


############## CLASS :: ConfigReader #####################
# Use this class to read WAS configuration inside        #
# RAFW                                                   #
#                                                        #
# THIS CLASS MAINTAINS STATE -- CANNOT BE A SINGLETON    #
#                                                        #
##########################################################
class ConfigReader:
	
	##########################################################
	## This constructor optionally defines ids which should not be read, 
	## and types which should be excluded
	##########################################################
	def __init__(self, importedIds=[], excludedTypes = [], excludedAttributes = []):
		self.importedIds = importedIds
		self.excludedTypes = excludedTypes
		self.processTypesFirst = []
		self.excludedAttributes = excludedAttributes
		self.LOGGER = _Logger("ConfigReader", MessageManager.RB_WEBSPHERE_WAS)
		# load the list of attributes which are treated as password fields
		self._updatePasswordAttributes()
		self.configTypeKeys = ConfigTypeKeys()
		## default to read passwords
		self.excludePasswords = 0
		self.allTypes = AdminConfig.types().split(newline)
		self.attrsForTypes = {}
	#endDef

	"""
	Sets the list of attributes which will not be read.  Attributes follow the form
	ConfigType.AttributeName so that attributes can be excluded specific to a Config Type
	
	Example: J2CResourceAdapter.deploymentDescriptor results in the attribute 
	deploymentDescriptor to be skipped when J2CResourceAdapter object types are imported.
	
	To include an attribute and override/set the runtime value 
	append =<value> to the attribute definition.
	
	@param excludedAttributes: The attributes to exclude or replace their values.
	                           To exclude: ['Security.wsPasswordEncryptions', ...]
	                           To replace: ['LDAPUserRegistry.serverPassword=value', ...]
	"""
	def setExcludedAttributes(self, excludedAttributes):
		self.excludedAttributes = excludedAttributes
	#endDef

	"""
		If set to true, passwords will not be read.  
		
		This is useful if you are using configReader to find objects to 
		update because password values are not used to locate objects.
	"""
	def setExcludePasswords(self, excludePasswords):
		self.excludePasswords = excludePasswords
	#endDef
	
	"""
		Set the ConfigTypeKeys object that contains the keys to use when writing Reference object
		attributes.
	"""
	def setConfigTypeKeys(self, configTypeKeys):
		self.configTypeKeys = configTypeKeys
	#endDef
	
	"""
		Get the ConfigTypeKeys object that contains the keys to use when writing Reference object
		attributes.  If more attributes should be excluded when displaying references, add them
		for the Config Type to the ConfigTypeKeys object returned.
	"""
	def getConfigTypeKeys(self):
		return self.configTypeKeys
	#endDef
			
	def _updatePasswordAttributes(self, newAttribute = None):
		Performance.start("ConfigReader._updatePasswordAttributes")
		propertyMgrProps = Globals.getInstallRoot() + "/product/lib/jython/PasswordMediator.properties"
		
		pmProps = SystemUtils.getPropertyDict(propertyMgrProps)
		passwordAttrStr = SystemUtils.readProperty(pmProps, "wasPasswordFields", "")
		self.passwordAttributes = passwordAttrStr.split(',')
		if (newAttribute is not None):
			# FileHelper depends on com.ibm.rational.rafw.shared.util.PropertiesUtil
			# which doesn't run in a Java 1.4 environment (like WAS6)
			# FileHelper.updatePropertyFile(propertyMgrProps, "wasPasswordFields", passwordAttrStr + "," + newAttribute)
			self.LOGGER.error("Attribute " + newAttribute + " not in list of password attributes.  ")
			self.LOGGER.error("   The value for this password will not be encrypted.")
		#endIf
		Performance.end("ConfigReader._updatePasswordAttributes")
	#endDef

	##########################################################
	## Determines if the supplied file contains a beginMarker and an endMarker
	## Returns 1 if the file is VALID and 0 otherwise
	##########################################################
	def processTypeFirst(self, configType):
		self.processTypesFirst.append(configType)
	#endDef
	
	##########################################################
	## Reads all data from WAS with the supplied id, then processes all attributes
	## that contain WAS ids into children. 
	##
	## Returns a dict with the following keys
	## id -- the type of the WAS config element referred to by the supplied id
	## attrs -- the attributes of the id, with all attributes referring to WAS ids removed
	## children 
	## An array of similar dicts, referring to each child referenced by this configuration
	## (each child contain the same:) 
	## ---id 
	## ---attrs
	## ---children
	##########################################################
	def showAll(self, id):
		self.LOGGER.traceEnter(id)
		if (str(id).split("#")[1].find(" ") >= 0):
			print "WARNING: ignored key = " + str(id)
			self.LOGGER.traceExit(str(None))
			return None
		#endIf
		# print "reading AllConfigData under: " + id
		# self.importedIds.append(id)
		subAttrs = self.show(id)
		children = []
		configElements = self.removeChildIdsToProcess(id, subAttrs)
		
		## process reference ids
		for childKey in configElements.referenceIds.keys():
			childIds = configElements.referenceIds[childKey]
			for childId in childIds:
				if (childId not in self.importedIds):
					child = {}
					childType = self.getWASType(childId)
					child['id'] = childType
					child['children'] = []
					childAttrs = self.show(childId)
					## remove all fields containing object ids
					self.removeChildIdsToProcess(childId, childAttrs)
					## add those attributes which are used to locate the object
					uniqueAttrs = self.configTypeKeys.getUniqueAttrs(childType)
					child['attrs'] = {}
					for uniqueAttr in uniqueAttrs:
						if (childAttrs.has_key(uniqueAttr)):
							child['attrs'][uniqueAttr] = childAttrs[uniqueAttr]
						#endIf
					#endFor
					if (len(child['attrs'].keys()) == 0):
						child['attrs'] = childAttrs
					#endIf
					child['attrs'][Globals.WAS_KEY_ATTR_NAME] = childKey
					child['attrs'][Globals.RAFW_TYPE] = Globals.RAFW_TYPE_REFERENCE
						
					children.append(child)
				#endIf
			#endFor
		#endFor
		## process higher priority children first
		for childKey in configElements.highPriorityIds.keys():
			childIds = configElements.highPriorityIds[childKey]
			for childId in childIds:
				child = self.processId(childId)
				if (child != None and child['id'] not in self.excludedTypes):
					child['attrs'][Globals.WAS_KEY_ATTR_NAME] = childKey
					children.append(child)
				#endIf
			#endFor
		#endFor
		## process regular priority children second
		#for childKey in childIds['none'].keys():
		for childKey in configElements.lowPriorityIds.keys():
			childIds = configElements.lowPriorityIds[childKey]
			for childId in childIds:
				child = self.processId(childId)
				if (child != None and child['id'] not in self.excludedTypes):
					child['attrs']['WASKey'] = childKey
					if (child['id'] != "builtin"):
						children.append(child)
				#endIf
			#endFor
		#endFor
		data = {}
		data['id'] = self.getWASType(id)
		data['attrs'] = subAttrs
		data['children'] = children
		self.LOGGER.traceExit(data)
		return data
	#endDef

	##########################################################
	## If the id hasn't already been processed, processes it using showAll
	## Else returns None
	##
	## @param subId: - the id to process
	##
	##########################################################
	def processId(self, subId):
		# print "prcessId: " + subId
		## if subId has not already been processed
		if (subId not in self.importedIds):
			return self.showAll(subId)
		#endIf
		return None
	#endDef
	
	##########################################################
	## Examines each attribute in the supplied data and determines if they contain
	## WAS configuration ids that need to be processed.
	##
	## Attributes can either refer to 1 WAS configuration object, 
	## or a list of many configuration objects.
	##
	## @param subAttrs: the attributes to process
	##
	## @return: configElements object with high and low priority ids
	##########################################################
	def removeChildIdsToProcess(self, id, subAttrs):
		configElements = ConfigElements()

		subKeys = subAttrs.keys()
		## get keys that are reference keys
		referenceKeys = self.getReferenceKeys(id)
		for subKey in subKeys:
			subId = subAttrs[subKey]
			subIds = AdminHelper.extractWASIdsFromList(subId)
			if (subKey in referenceKeys):
				configElements.addReferenceIds(subKey, subIds)
			else:
				for anId in subIds:
					if (self.getWASType(anId) in self.processTypesFirst):
						configElements.addHighPriorityId(subKey, anId)
					else:
						configElements.addLowPriorityId(subKey, anId)
					#endIf
				#endFor
			#endIf
			if (len(subIds) > 0):
				# an item was found, remove the key from attrs
				del subAttrs[subKey]
			#endIf
		#endFor
		return configElements
	#endDef
	
	
	
	##########################################################
	## 
	## Reads WAS configuration data under the supplied scope for 
	## the requested configuration type.
	##
	## This method is not called recursively like the showAll method
	## This method resets the importedIds variable
	##
	## @param scope: The parent scope of the configuration to read
	## @param scopeType: cell/node/cluster/server - used to filter out 
	##                   elements not in the correct scope 
	## @param configType: The data type to read - ex: JDBCProvider - this is generally the 
	##                    xml:id in the WAS configuration files 
	## @param excludedTypes: The types to exclude. ex: ['JAASAuthData', ...] 
	##
	##########################################################
	def readConfigData(self, scope, scopeType, configType, excludedTypes = [], excludedAttributes = None):
		self.LOGGER.traceEnter([scope, scopeType, configType, excludedTypes, excludedAttributes])
		scopeId = AdminConfig.getid(scope)
		returnVal = self.readConfigDataUsingParentId(scopeId, scopeType, configType, excludedTypes, excludedAttributes)
		self.LOGGER.traceExit(returnVal)
		return returnVal
	#endDef

	##########################################################
	## 
	## Reads WAS configuration data under the supplied scope for 
	## the requested configuration type.
	##
	## This method is not called recursively like the showAll method
	## This method resets the importedIds variable
	##
	## @param scopeid: The parent scope id of the configuration to read
	## @param scopeType: cell/node/cluster/server - used to filter out 
	##                   elements not in the correct scope 
	## @param configType: The data type to read - ex: JDBCProvider - this is generally the 
	##                    xml:id in the WAS configuration files 
	## @param excludedTypes: The types to exclude. ex: ['JAASAuthData', ...] 
	##
	##########################################################
	def readConfigDataUsingParentId(self, scopeId, scopeType, configType, excludedTypes = [], excludedAttributes = None):
		Performance.start("ConfigReader.readConfigDataUsingParentId");
		if (excludedAttributes is not None):
			self.LOGGER.warn("The fifth parameter to readConfigDataUsingParentId has been deprecated.  " 
							+ "Please set attributes to exclude on the ConfigReader instance.")
			self.excludedAttributes = excludedAttributes
		#endIf
		data = []
		if (len(scopeId) == 0):
			msg1 = "ERROR: unable to find parent scope: " + scopeId
			msg2 = "Cannot read config data."
			self.LOGGER.error(msg1)
			self.LOGGER.error(msg2)
			raise Exception (msg1 + msg2)
			Performance.end("ConfigReader.readConfigDataUsingParentId");
			return data
		#endIf

		for objId in AdminConfig.list(configType, scopeId).split(newline):
			self.importedIds = []
			## if anything found, is in scope, and not builtin
			if (len(objId) > 0 and AdminHelper.isInScope(scopeType, objId) and (objId.find("builtin") == -1)):
				## never show the top level configuration element in references
				if (excludedTypes != []):
					match = "no"
					for excludedType in excludedTypes:
						if (SystemUtils.regexp(excludedType, objId) == 1):
							match = "yes"
						#endIf
					#endFor
					if (match == "no"):
						#self.LOGGER.info("Importing " + objId)
						self.importedIds.append(objId)
						data.append(self.showAll(objId))
					#endIf
				else:
					#self.LOGGER.info("Importing " + objId)
					self.importedIds.append(objId)
					data.append(self.showAll(objId))
				#endIf
			#endIf
		#endFor
		Performance.end("ConfigReader.readConfigDataUsingParentId");
		return data
	#endDef
	
	##########################################################
	## Removes xml unfriendly characters from the supplied value.
	##
	## @param propValue: The property value to clean
	## @return: The supplied input string without [] " etc.
	##
	##########################################################
	def cleanXMLString(self, propValue):
		self.LOGGER.traceEnter(propValue)
		# print "cleaning: " + propValue
		## Clean [ at beginning and ] at end
		if (propValue.endswith(']')):
			propValue = propValue[1:len(propValue)-1]
		#endIf
		## Clean surrounding "
		if propValue.startswith('"') and propValue.endswith('"'):
			propValue = propValue[1:len(propValue)]
			propValue = propValue[0:len(propValue)-1]
		#endIf
		## If propValue is a WAS identifier Clean all occurences of " 
		if AdminHelper.extractWASIdsFromList(propValue) != []:
			indexOfQuote = propValue.find('"')
			while (indexOfQuote != -1):
				propValue = propValue[0:indexOfQuote] + propValue[indexOfQuote + 1:len(propValue)]
				indexOfQuote = propValue.find('"')
			#endWhile
		#endIf
		## Clean all occurence of [Ljava.lang.String;
		if (propValue == "[Ljava.lang.String;"):
			propValue = "java.lang.String"
		#endIf
		self.LOGGER.traceExit(propValue)
		return propValue
	#endDef

	##########################################################
	## Converts a string in the form {a=1, b=2} into a java Map 
	##########################################################
	def convertToMap(self, values):
		data = HashMap()
		if (values.startswith('{')):
			values = values.split('{')[1]
		#endIf
		if (values.endswith('}')):
			values = values.split('}')[0]
		#endIf
		for elem in values.split(', '):
			if (len (elem) > 0):
				keyValue = elem.split('=', 1)
				if (len(keyValue) == 1):
					## the value is blank ie foo=
					data.put(keyValue[0].strip(), "")
				elif (len(keyValue) == 2):
					data.put(keyValue[0].strip(), keyValue[1].strip())
			#endIf
		#endFor
		return data
	#endDef
	
	##########################################################
	## Converts a string in the form {a=1, b=2} into a Dict 
	##########################################################
	def convertToDict(self, values):
		data = {}
		if (values.startswith('{')):
			values = values.split('{')[1]
		#endIf
		if (values.endswith('}')):
			values = values.split('}')[0]
		#endIf
		for elem in values.split(', '):
			if (len (elem) > 0):
				keyValue = elem.split('=', 1)
				if (len(keyValue) == 1):
					## the value is blank ie foo=
					data[keyValue[0].strip()] = ""
				elif (len(keyValue) == 2):
					data[keyValue[0].strip()] = keyValue[1].strip()
			#endIf
		#endFor
		return data
	#endDef
	
	def typeSupportsDescription(self, type):
		if not ( type in self.attrsForTypes.keys()):
			attrs = AdminConfig.attributes(type).split(newline)
			self.attrsForTypes[type] = 0
			for attr in attrs:
				if (len(attr) > 0) :
					(key, ino) = attr.split(' ', 1)
					if key == 'description':
						self.attrsForTypes[type] = 1
						break
					#endIf
				#endif
			#endFor
		#endIf
		return self.attrsForTypes[type]
	#endDef

	#checks to see if the WAS 'type' supports a description attribute and if it does
	#but we didnt get a value for it cause its empty, lets ensure we have a value of '' for it
	def fixConfigDictDesc(self, configDict, type):
		if type == 'builtin':
			return configDict 
		typeSupportsDesc = self.typeSupportsDescription(type)
		didntStoreDesc = 'description' not in configDict.keys()
		if (typeSupportsDesc and didntStoreDesc):
			configDict['description'] = ""
		#endIf
		return configDict
	#endDef
	
	##########################################################
	##  Extends AdminConfig.show to display the values as a Dict
	## @param objId: the object id to show 
	##########################################################
	def show(self, objId):
		self.LOGGER.traceEnter(objId)
		wasType = self.getWASType(objId)
		try:
			wasAttrs = AdminConfig.show(objId).split(newline)
			returnDict = self.convertAdminConfigAttrsToDict(objId, wasAttrs)
			returnDict = self.fixConfigDictDesc(returnDict, wasType)
			self.LOGGER.traceExit(returnDict)
			return returnDict
		except:	
			self.LOGGER.error("unable to show: " + objId)
			self.LOGGER.error(" Details are: " + str(sys.exc_info()[0]) + ":" + str(sys.exc_info()[1]))
			#traceback.print_tb(sys.exc_info()[2])
			if (objId.find(':') != -1):
				print "Trying again without :"
				newObjId = objId.split(':', 1)[1]
				try:
					wasAttrs = AdminConfig.show(newObjId).split(newline)
					returnDict = self.convertAdminConfigAttrsToDict(newObjId, wasAttrs)
					returnDict = self.fixConfigDictDesc(returnDict, wasType)
					self.LOGGER.traceExit(returnDict)
					return returnDict
				except Exception, ex:	
					print "ERROR: unable to show: " + newObjId
					raise ex
				#endTry
			#endIf
			returnDict = {}
			self.LOGGER.traceExit(returnDict)
			return returnDict
		#endTry
	#endDef
	
	##########################################################
	## Converts the string returned by WAS AdminConfig.show to a Dict
	## All strings are cleaned for XML compatibility.
	##
	## Any attribute name in self.excludedTypes will be excluded
	##
	## @param objId: The id which contains these attributes
	## @param adminAttrs: The attribute string returned by WAS
	## @see: cleanXMLString
	## @return: A Dict containing attribute name=value pairs
	##########################################################
	def convertAdminConfigAttrsToDict(self, objId, adminAttrs):
		self.LOGGER.traceEnter([objId, adminAttrs])
		data = {}
		processedAttrs = []
		currValue = ""
				
		for adminAttr in adminAttrs:
			if ( adminAttr.startswith('[') and adminAttr.endswith(']') ):
				processedAttrs.append(adminAttr)
			elif ( adminAttr.startswith('[') ):
				currValue = adminAttr
			elif ( adminAttr.endswith(']') ):
				processedAttrs.append(currValue + adminAttr)
			else:
				currValue = currValue + adminAttr
			#endIf
		#endFor
		
		self.LOGGER.traceEnter([adminAttrs])
		
		for adminAttr in processedAttrs:
			if ( adminAttr.find('\n') != -1 or adminAttr.find('\r') != -1 ):
				adminAttr = adminAttr.replace('\r',' ')
				adminAttr = adminAttr.replace('\n',' ')
				print "Replacing new line characters with spaces from objectId " + str(objId)
				print "Attribute: " + str(adminAttr)
			#endIf
		
			adminAttr = adminAttr[1:len(adminAttr)-1]
			self.LOGGER.trace("adminAttr after removing ends:" + str(adminAttr))
			name, value = adminAttr.split(" ", 1)
			
			## check for attributes to be excluded or get their values replaced
			attributeIdentifier = self.getWASType(objId) + "." + name
			excludeMe = 0
			for excludedAttribute in self.excludedAttributes:
				array_ = excludedAttribute.split("=")
				if len(array_) > 1:
					attributeName, replaceValue = array_
				else:
					attributeName, replaceValue = array_[0], None
				#endIf
				if (attributeName == attributeIdentifier):
					if replaceValue is not None:
						## include this attribute and replace it's value
						value = replaceValue
					else:
						## exclude this attribute
						excludeMe = 1
					#endIf
				#endIf
			#endFor
			## end checking excluded attributes subroutine
			
			if ((name not in self.excludedTypes) and (not excludeMe)):
				# if the value is *****, then this is a password attribute value
				if ( (value == "*****") or ( name in self.passwordAttributes ) ):
					if (not self.excludePasswords):
						if (name not in self.passwordAttributes):
							self._updatePasswordAttributes(name)
						#endIf
						value = self.readPassword(objId, name)
						if (value == "None"):
							print "Warning: Unable to read password value for attribute"
							print "         attribute '" + name + "' will not be imported"
						else:
							data[name] = self.cleanXMLString(value)
					#endIf
				else:		
					data[name] = self.cleanXMLString(value)
			#endIf
		#endFor
		self.LOGGER.traceExit(data)
		return data
	#endDef
	
	
	##########################################################
	## Certain elements appear as children, but are really hints as to an elements parent
	## example of this are JDBCProvider and JMSProvider
	## This method finds these elements given by wasKeyName and 
	## simplifies them by removing children, attributes and changing the id
	##
	## @param data: Array of elements, each element's children are consulted
	## @param wasKeyName: The value stored in Globals.WAS_KEY_ATTR_NAME which is used to match the element
	## @param newId: The new ID of the matched element or None if the existing id should be left as is.
	## @param attrKeysToKeep: The attribute names to keep for the element, 
	##                        all others are removed
	##########################################################
	def stripElement(self, data, wasKeyName, newId, attrKeysToKeep):
		for element in data:
			for child in element['children']:
				if (child['attrs'].has_key(Globals.WAS_KEY_ATTR_NAME) and child['attrs'][Globals.WAS_KEY_ATTR_NAME] == wasKeyName):
					child['children'] = []
					if (newId is not None):
						child['id'] = newId
					#endIf
					newAttrs = {}
					for attrKeyToKeep in attrKeysToKeep:
						if (child['attrs'].has_key(attrKeyToKeep)):
							keyValue = child['attrs'][attrKeyToKeep]
							newAttrs[attrKeyToKeep] = keyValue
					#endFor
					child['attrs'] = newAttrs
				#endIf
			#endFor
		#endFor
	#endDef
		

	
	##########################################################
	## Parse an id string to find the type in the order
	## Garbage#JDBCProvider_Garbage
	##
	## @return: The above example would return JDBCProvider
	##
	##########################################################
	def getWASType(self, wasid):
		self.LOGGER.traceEnter(wasid)
		if (wasid == None or wasid.find('#') == -1):
			raise ConfigReaderException("Configuration wasid is none or invalid while determining WAS configuration type: " + str(wasid))
		tmptype = wasid.split("#")[1]
		tmptype = tmptype.split("_")[0]
		tmptype = tmptype.split(")")[0]
		
		if (tmptype not in self.allTypes):
			if (wasid.find("NA_WSN_1") > -1):
				#set type to this as this is the type it should be
				tmptype = "VariableSubstitutionEntry"
			elif (wasid.startswith("Default") and wasid.find("_WC") !=-1):
				tmptype = "WorkClass"
			elif (wasid.find("healthclass.xml") > -1):
				tmptype = "HealthClass"
			elif (wasid.find("serviceclass.xml") > -1):
				tmptype = "ServiceClass"
			elif (wasid.find("CustomProperty") > -1):
				tmptype = "Property"
			elif (wasid.find("_smtps") > -1 or wasid.find("_pop3s") > -1 or wasid.find("_imaps") > -1 or 
					wasid.find("_smtp") > -1 or wasid.find("_pop3") > -1 or wasid.find("_imap") > -1):
				tmptype="ProtocolProvider"
			#else:
			#	print "whoami? " + str(type) + ", wasid: " + str(wasid)
			#end if
		#endIf

		self.LOGGER.traceExit(tmptype)
		return tmptype
	#endDef
	
	"""
		Determines which attributes contained in the supplied id are defined
		to contain references to other WebSphere ids.  
	
		Reference ids are WebSphere ids that exist outside the parent-child
		hierarchy.  As such, they must be handled differently than ids which are children
		of a given WebSphere configuration element.  
	
		@param id: The parent object whose attributes should be consulted
		@return: a list of attribute names that are defined to contain references to 
		         non-sibling WebSphere configuration elements
	"""
	def getReferenceKeys(self, id):
		Performance.start("ConfigReader.getReferenceKeys");
		referenceKeys = []
		if (len(id) > 0 and (id.find("builtin") == -1)):
			wasType = self.getWASType(id)
			for attribute in AdminConfig.attributes(wasType).split(newline):
				if (len (attribute) > 0):
					(key, info) = attribute.split(' ', 1)
					if (info.endswith('@') or info.endswith('@*')):
						referenceKeys.append(key)
					#endIf
				#endIf
			#endFor
		#endIf
		Performance.end("ConfigReader.getReferenceKeys");
		return referenceKeys
	#endDef
	
	"""
		Determines which attributes contained in the supplied id are defined
		to contain lists of other WebSphere ids.  
	
		Child ids are WebSphere ids that are children of the parent type.
	
		@param id: The parent object whose attributes should be consulted
		@param excludedAttributes: List of attribute names to exclude in the form 
		                           <type>.<attribute> aka: Security.keyManagers
		@return: a list of attribute names that are defined to contain lists of child 
		         WebSphere configuration elements
	"""
	def getChildrenKeys(self, id, excludedAttributes):
		Performance.start("ConfigReader.getChildrenKeys");
		childrenKeys = []
		if (len(id) > 0 and (id.find("builtin") == -1)):
			wasType = self.getWASType(id)
			for attribute in AdminConfig.attributes(wasType).split(newline):
				(key, info) = attribute.split(' ', 1)
				## attributes contains child ids, but not references to non-children ids
				if (info.endswith('*') and not info.endswith('@*')):
					if (wasType + "." + key not in excludedAttributes):
						childrenKeys.append(key)
					#endIf
				#endIf
			#endFor
		#endIf
		Performance.end("ConfigReader.getChildrenKeys");
		return childrenKeys
	#endDef
	
	"""
		returns 1 if the was object contains the supplied attribute,
		else 0
	"""
	def hasAttr(self, id, attr):
		Performance.start("ConfigReader.hasAttr");
		if (len(id) > 0 and (id.find("builtin") == -1)):
			wasType = self.getWASType(id)
			for attribute in AdminConfig.attributes(wasType).split(newline):
				(key, info) = attribute.split(' ', 1)
				if (key == attr):
					Performance.end("ConfigReader.hasAttr");
					return 1
				#endIf
			#endFor
		#endIf
		Performance.end("ConfigReader.hasAttr");
		return 0
	#endDef
	
	"""
		returns 1 if the WAS object contains all of the supplied attributes,
		else 0
	"""	
	def hasAllAttrs(self, id, attrs):
		self.LOGGER.traceEnter([id, attrs])
		if (len(id) > 0):
			wasType = self.getWASType(id)
			if (wasType.find("builtin") == -1):
				## find the attributes that exist on this wasType
				wasAttrs = []
				for attribute in AdminConfig.attributes(wasType).split(newline):
					(key, info) = attribute.split(' ', 1)
					wasAttrs.append(key)
				#endFor
				for attr in attrs:
					if (attr not in wasAttrs):
						self.LOGGER.traceExit(0)
						return 0
					#endIf
				#endFor
				self.LOGGER.traceExit(1)
				return 1
			else:
				self.LOGGER.trace("Attributes cannot be retrieved for <builtin>  Cannot determine if this object has all attributes.")
				self.LOGGER.traceExit(0)
				return 0
			#endIf
		#endIf
		self.LOGGER.traceExit(0)
		return 0
	#endDef
	
	##########################################################
	##
	## Reads a password value using ConfigService
	## else 0
	##
	## @param id: The id which contains the attribute
	## @param adminAttrs: The name of the attribute which has the password value
	##
	##########################################################
	def readPassword(self, id, attrName):
		self.LOGGER.traceEnter([id, attrName])
		id = self.stripQuotes(id)
		if (id.find('(') > 0):
			id = id[id.find('('):len(id)]
			self.LOGGER.debug("id contains ( and ), extracted WAS id from string is: " + str(id))
		id = self.stripParens(id)
		configService = ConfigServiceFactory.getConfigService()
		if (configService is None):
			props = java.util.Properties(); 
			props.setProperty("location", "local");
			configService = ConfigServiceFactory.createConfigService(1, props); 
		#endIf
		session = com.ibm.websphere.management.Session()
		
		wasConfigType = self.getWASType(id)
		objectNameMatches = configService.resolve(session, wasConfigType)
		# find the object who's id matches
		attrValue = None
		for objectName in objectNameMatches:
			thisConfigId = objectName.getKeyProperty('_Websphere_Config_Data_Id')
			self.LOGGER.debug("comparing ids found:" + str(thisConfigId) + " match against:" + str(id))
			if (thisConfigId == id):
				self.LOGGER.debug("resolved object " + str(objectName))
				attrValue = configService.getAttribute(session, objectName, attrName)
			#endIf
		#endFor
		configService.discard(session)
		self.LOGGER.traceExit(attrValue)
		return str(attrValue)
	#endDef

	"""
		stripParens(id) 
		Remove ( and ) from supplied string         
	"""
	def stripParens(self, id):
		self.LOGGER.traceEnter(id)
		if (id[0] == '('):
			id = id[1:len(id)]
		if (id[len(id)-1] == ')'):
			id = id[0:(len(id)-1)]
		self.LOGGER.traceExit(id)
		return id
	#endDef
	
	"""
		stripQuotes(id) 
		Remove " and " from supplied string if they exist       
	"""
	def stripQuotes(self, id):
		self.LOGGER.traceEnter(id)
		if (id[0] == '"'):
			id = id[1:len(id)]
		if (id[len(id)-1] == '"'):
			id = id[0:(len(id)-1)]
		self.LOGGER.traceExit(id)
		return id
	#endDef

	def findByParentId(self, xmlNode, scopeid = None):
		parentName = xmlNode.getAttrValue(Globals.RAFW_PARENT_NAME)
		parentType = xmlNode.getAttrValue(Globals.RAFW_PARENT_TYPE)
		if (scopeid is not None) :
			possibleParents = AdminConfig.list(parentType, scopeid).split(newline)
		else: 
			possibleParents = AdminConfig.list(parentType).split(newline)
		#endIf
		dataDict = None
		for possParent in possibleParents:
			subAttrs = self.show(possParent)
			if subAttrs['name'] == parentName:
				#found my parent...lets get its children
				dataDict = self.show(possParent)
				break
			#endIf
		#endFor
		referenceables = ""
		if newline.find("\n") != -1:
			sep = "\n"
		else:
			sep = "\r"
		if dataDict is not None:
			#mimicing the output of AdminConfig.list()
			referenceables = sep.join(dataDict['referenceables'].split(' '))
		#endIf
		return referenceables
	#endDef
	
	"""
		Finds object ids which are of the same type as the supplied xmlNode
		and contains values for the same attributes in the xmlNode.
		
		Blank attribute values are ignored.  
	
		The search for objects can be contained within the supplied scope
			
		A list of WAS objectIds is returned 
		If none are found an empty list is returned.
		
	"""
	def findObjectIds(self, xmlNode, scopeId = None, mgmtScopeName = None):
		self.LOGGER.traceEnter([xmlNode, scopeId])
		configType = xmlNode.getNodeNameFixed()
		scopeName = None
		if (xmlNode.getNodeNameFixed() == "Referenceable" and xmlNode.hasAttr(Globals.RAFW_PARENT_NAME) 
				and xmlNode.hasAttr(Globals.RAFW_PARENT_TYPE)):
			objIds = self.findByParentId(xmlNode,scopeId)
			if (scopeId is not None):
				scopeName = AdminHelper.getScope(scopeId)
		elif (scopeId is None):
			objIds = AdminConfig.list(configType)
		else:
			objIds = AdminConfig.list(configType, scopeId)
			scopeName = AdminHelper.getScope(scopeId)
		#endIf
		matchedIds = []
		self.LOGGER.trace("found these possible matches:" + str(objIds))
		for objId in objIds.split(newline):
			if (len(objId) > 0):
				## only consider this a match if the config types match 
				## (we didn't get a sub-class in the list query above)
				wasType = self.getWASType(objId)
				if ((configType=="TrustManager" or configType=="KeyManager") and configType == wasType):
					wasAttrs = self.show(objId)
					if (self._compareIgnoringBlanks(xmlNode.buildNodeAttrsDict(), wasAttrs)):
						if mgmtScopeName != None and len(mgmtScopeName) > 0:
							mgmtScope = wasAttrs['managementScope']
							mgmtScopeAttrs = self.show(mgmtScope)
							if mgmtScopeAttrs['scopeName'] == mgmtScopeName:
								self.LOGGER.trace("found a match on:" + str(objId))
								matchedIds.append(objId)
							#endIf
						else:
							self.LOGGER.trace("found a match on:" + str(objId))
							matchedIds.append(objId)
						#endIf
				elif (configType == wasType):
					if (scopeName is None or AdminHelper.isInScope(scopeName, objId)):
						wasAttrs = self.show(objId)
						## match the attributes values which were passed in
						## return the objId if we have a match
						if (self._compareIgnoringBlanks(xmlNode.buildNodeAttrsDict(), wasAttrs)):
							self.LOGGER.trace("found a match on:" + str(objId))
							matchedIds.append(objId)
						#endIf
					#endIf
				#endIf
			#endIf
		#endFor
		self.LOGGER.traceExit(str(matchedIds))
		return matchedIds
	#endDef

	"""
		Gets the id of the configuration type supplied under the defined parent.
		
		WASKey must be supplied as an attribute of the xmlNode to 
		read the existing ids for the key on the parent object.
	
		If only 1 id exists for the WASKey in the configuration, that ID is returned
		
		If more than 1 id is set then the xmlNode is compared against all ids to find the 
		one that matches.  
		
		If the attributes of more than 1 configuration object match the attributes in 
		the supplied XML node, returns None
		
		If no ids are set for the WASKey in the configuration, returns None
		 
		@raise ConfigReaderException: If the supplied node does not have a WASKey attribute
	"""
	def getObjectId(self, xmlNode, parentId):
		self.LOGGER.traceEnter([xmlNode, parentId])
		if (xmlNode.hasAttr(Globals.WAS_KEY_ATTR_NAME)):
			configType = xmlNode.getNodeNameFixed()
			wasKey = xmlNode.getAttrValue(Globals.WAS_KEY_ATTR_NAME)
			wasKeyValue = AdminConfig.showAttribute(parentId, wasKey)
			objectIds = AdminHelper.extractWASIdsFromList(wasKeyValue)
			## if there is only 1, no need to search for it
			if (len(objectIds) == 1):
				self.LOGGER.traceExit(str(objectIds[0]))
				return objectIds[0]
			#endIf
			matchedIds = []
			self.LOGGER.trace("found these possible matches:" + str(objectIds))
			for objId in objectIds:
				wasType = self.getWASType(objId)
				if (configType == wasType):
					wasAttrs = self.show(objId)
					## match the attributes values which were passed in
					## return the objId if we have a match
					if (self._compareIgnoringBlanks(xmlNode.buildNodeAttrsDict(), wasAttrs)):
						self.LOGGER.trace("found a match on:" + str(objId))
						matchedIds.append(objId)
					#endIf
				#endIf
			#endFor
			if (len(matchedIds) == 1):
				self.LOGGER.traceExit(str(matchedIds[0]))
				return matchedIds[0]
			else:
				self.LOGGER.traceExit(str(None))
				return None
			#endIf
		else:
			raise ConfigReaderException(Globals.WAS_KEY_ATTR_NAME 
					+ " is not defined as an attribute of xmlNode: " + str(xmlNode))
		#endIf
	#endDef
	
	def _compareIgnoringBlanks(self, primaryAttrs, matchAttrs, keysToMatch = None):
		self.LOGGER.traceEnter([primaryAttrs, matchAttrs, keysToMatch])
		if (keysToMatch is None):
			keysToMatch = primaryAttrs.keys()
		#endIf
		for primaryKey in keysToMatch:
			self.LOGGER.trace("Matching on key: " + primaryKey)
			## each key in primaryAttrs must exist in matchAttrs and the value must be the same
			## ignoring keys with empty values and RAFW keys
			if (primaryKey == Globals.RAFW_TYPE or primaryKey == Globals.WAS_KEY_ATTR_NAME or
					primaryKey == Globals.RAFW_PARENT_TYPE or primaryKey == Globals.RAFW_PARENT_NAME):
				self.LOGGER.trace("Ignoring a match on key: " + primaryKey + " because it is not a WAS attribute")
				continue
			elif (not primaryAttrs.has_key(primaryKey) or primaryAttrs[primaryKey] is None or len (primaryAttrs[primaryKey]) == 0):
				self.LOGGER.trace("Ignoring a match on key: " + primaryKey + " because the search criteria does not contain this key")
				continue
			elif (matchAttrs.has_key(primaryKey) and matchAttrs[primaryKey] is not None):
				## match has the key, compare the values
				if (primaryAttrs[primaryKey] == matchAttrs[primaryKey]):
					continue
				else:
					self.LOGGER.traceExit(0)
					return 0
			else:
				## the matchAttrs doesn't contain the key, objects don't match
				self.LOGGER.trace("WAS config is missing a value for key: " + primaryKey + " objects to not match")
				self.LOGGER.traceExit(0)
				return 0
			#endIf
		#endFor
		self.LOGGER.traceExit(1)
		return 1	
	#endDef
	
#endClass



############ CLASS :: XMLConfigReader ####################
# Use this class to read RAFW configuration in the same  #
# form as the ConfigReader class, except this reads      #
# from an XML file, while the ConfigReader reads from    #
# WebSphere                                              #
#                                                        #
##########################################################
class XMLConfigReader:
	
	def __init__(self):
		self.LOGGER = _Logger("XMLConfigReader", MessageManager.RB_WEBSPHERE_WAS)
	#endDef
	
	def readXmlConfig(self, xmlPropNodeArray):
		self.LOGGER.traceEnter(xmlPropNodeArray)
		data = []
		for xmlNode in xmlPropNodeArray:
			child = {}
			child['id'] = xmlNode.getNodeNameFixed()
			child['attrs'] = xmlNode.buildNodeAttrsDict()
			child['children'] = self.readXmlConfig(xmlNode.getChildrenArray())
			data.append(child)
		#endFor
		self.LOGGER.traceExit(data)
		return data
	#endDef
		
	def readXmlWsadminConfig(self, xmlPropNodeArray):
		self.LOGGER.traceEnter(xmlPropNodeArray)
		data = ArrayList()
		for xmlNode in xmlPropNodeArray:
			child = WsadminConfig()
			child.setId(xmlNode.getNodeNameFixed())
			child.setAttributes(xmlNode.buildNodeAttrsMap())
			child.setChildren(self.readXmlWsadminConfig(xmlNode.getChildrenArray()))
			data.add(child)
		#endFor
		self.LOGGER.traceExit(data)
		return data
	#endDef
#endClass

"""
CLASS :: ConfigTypeKeys

Stores the Keys that are used to search for an modify Websphere configuration by
ConfigType.  The default keys can be modified when necessary by scripts.
"""
class ConfigTypeKeys:
	
	def __init__(self):
		Performance.start("ConfigTypeKeys.__init__")
		self.LOGGER = _Logger("ConfigTypeKeys", MessageManager.RB_WEBSPHERE_WAS)
		self.uniqueAttrsByConfigType = {}
		configTypeKeys = SystemUtils.getPropertyDict(
			Globals.getInstallRoot() + "/product/lib/jython/wsadmin_lib/ConfigTypeKeys.properties")
		for key in configTypeKeys.keys():
			self.setUniqueAttrs(key, configTypeKeys[key].split(","))
		#endFor
		Performance.end("ConfigTypeKeys.__init__")
	#endDef

	"""
		Use this method to specify the attribute which is used to identify 
		the configuration in WAS and match it to the configuration in RAFW
		for a given configuration type.
		
		The attribute name is used to match configuration on update.
		
		@param type -- The WAS Configuration Type
		@param attrs -- A list of attributes to match.  
		More than 1 attribute in the list indicates a composite key
	"""
	def setUniqueAttrs(self, type, attrs):
		if (len(type) > 0):
			self.uniqueAttrsByConfigType[type] = attrs
		else:
			self.LOGGER.debug("invalid arguments supplied: type=" + type + ", attrs=" + str(attrs))
		#endIf
	#endDef
	
	"""
		Returns the list of unique attributes that have been set for the 
		WAS config type.  
		
		If nothing has been set returns the 1 element list containing 'name'
	"""
	def getUniqueAttrs(self, type):
		self.LOGGER.traceEnter([type])
		if (self.uniqueAttrsByConfigType.has_key(type)):
			uniqueAttrs = self.uniqueAttrsByConfigType[type]
			self.LOGGER.traceExit([uniqueAttrs])
			return uniqueAttrs
		else:
			## default to name
			self.LOGGER.traceExit(["name"])
			return ["name"]
		#endIf
	#endDef	
	
#endClass
