"""
	Licensed Materials - Property of IBM Corp. 
IBM Rational Automation Framework 
(c) Copyright IBM Corporation 2003, 2012. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 

	
	File name: ConfigWriter.py
	
	This contains methods and classes to facilitate 
	writing WebSphere configuration data.
"""

from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager
from com.ibm.rational.rafw.wsadmin.performance import Performance
from ConfigReader import *
from AdminHelper import AdminHelper

import AdminConfig
import Globals

"""
CLASS :: ConfigWriter 
	Use this class to write WAS configuration from inside  
	RAFW                                                   
                                                        
	THIS CLASS MAINTAINS STATE -- CANNOT BE A SINGLETON    
"""
class ConfigWriter:

	"""
	"""
	def __init__(self):
		self.updatedIds = []
		self.configReader = ConfigReader()
		self.configReader.setExcludePasswords(1)
		self.LOGGER = _Logger("ConfigWriter", MessageManager.RB_WEBSPHERE_WAS)
		self.updateReferences = 0
		self.missingReferenceIdHander = None
		self.configTypeKeys = ConfigTypeKeys()
		## The top level scope id of manipulated config objects, 
		## Since a lot of recursion exists in this class, this variables
		## represents the scope id at the start.
		## set when createWASObject, updateWASObject, updateWASChildren 
		## and updateWASReferenceAttributes are invoked.
		self.initialScopeId = None
	#endDef
	
	def setConfigReader(self, configReader):
		""" 
			Sets the config reader object that will be used by this config writer
			overriding the default reader.
		"""
		self.configReader = configReader
	#endDef
	
	def getConfigReader(self):
		""" 
			Gets the config reader object that is being used by this config writer
		"""
		return self.configReader
	#endDef

	"""
		set updateReferences to true (1) if you want to locate and update references 
		when updating or creating objects in the WebSphere configuration
		
		If you want to update references after config has been created or updated
		use updateWASReferenceAttributes.  (This latter method is recommended.)
		
		When xmlNodes contain the attribute Globals.RAFW_TYPE="reference" then the other
		attributes in the xmlNode are used to search for matching WAS configuration.
		When a match is found, the id of the located object is set in the parent object
		under the attribute given by the WASKey attribute on the xmlNode.
		
		Defaults to false (0).
	"""
	def setUpdateReferences(self, updateReferences):
		self.LOGGER.info("References will be updated when configuration is modified or created")
		self.updateReferences = updateReferences
	#endDef

	"""
		Set the ConfigTypeKeys object that contains the keys to use when locating ConfigTypes
		to update.
	"""
	def setConfigTypeKeys(self, configTypeKeys):
		self.configTypeKeys = configTypeKeys
	#endDef
	
	"""
		Get the ConfigTypeKeys object that contains the keys used when locating ConfigTypes
		to update.
	"""
	def getConfigTypeKeys(self):
		return self.configTypeKeys
	#endDef
	
	"""
		Sets a handler which can be used to resolve cases where a reference id cannot be found.
	
		When the reference id is not found, this method is called:
		handleMissingReferenceId(xmlNode)
		
		This method can return the object id that should be used 
		for the reference or return None to indicate that the default 
		behavior should be used.  The default behavior is to throw an
		exception.
	"""
	def setMissingReferenceIdHander(self, missingReferenceIdHander):
		self.missingReferenceIdHander = missingReferenceIdHander
	#endDef
	
	"""
		Creates WAS configuration objects following the given pattern:
		The object type to create is given by the name of the xmlNode supplied.
		The attributes to use to create the object are given by 
		the attributes in the xmlNode
		The xmlNodes that are children of this node are created 
		recursively using the same pattern
	
		By default, this method will never attempt to write a "builtin" config element
		
		As all objects are created, the objectId is set in the xmlNode as configId
	
		@param xmlNode: The xmlNode to use for the data to write
		@param parentId: The id of the configuration data's parent
		@param excludedTypes: An array of WAS configuration types which should 
	                          not be written, these nodes are ignored
		@return: The id of the newly created object

		@attention: This method would be better named create -- since it creates objects that 
		            don't exist including children
		            It is analogous to modify -- which modifies an existing object as well as 
		            all of the objects children.
		            This method will fail if the object already exists and modify will fail 
		            if the object does not exist.
	"""
	def createWASObject(self, xmlNode, parentId, excludedTypes = ['builtin']):
		self.LOGGER.traceEnter([xmlNode, parentId, excludedTypes])
		self.initialScopeId = parentId
		newId = self._createWASObject(xmlNode, parentId, excludedTypes)
		self.LOGGER.traceExit(newId)
		return newId
	#endDef
	
	"""
		This method is called recursively by other methods in this class
	"""
	def _createWASObject(self, xmlNode, parentId, excludedTypes = ['builtin']):
		self.LOGGER.traceEnter([xmlNode, parentId, excludedTypes])
		type = xmlNode.getNodeNameFixed()
		if (xmlNode.hasAttr(Globals.RAFW_TYPE) and xmlNode.getAttrValue(Globals.RAFW_TYPE) == Globals.RAFW_TYPE_REFERENCE):
			## this should not happen unless there is an error and the xmlNode 
			## is not properly handled as a reference.
			self.LOGGER.error("References will not be created for xmlNode: " + type)
			self.LOGGER.traceExit()
			return
		#endIf
		newId = None
		if (type not in excludedTypes):
			self.LOGGER.info("creating: " + type + " under parent: " + parentId)
			newId = self.createEntity(xmlNode, parentId, excludedTypes)
			configElements = ConfigElements()
			for xmlChild in xmlNode.getChildrenArray():
				if (xmlChild.getNodeNameFixed() not in excludedTypes):
					## reference node
					if (xmlChild.hasAttr(Globals.RAFW_TYPE) and xmlChild.getAttrValue(Globals.RAFW_TYPE) == Globals.RAFW_TYPE_REFERENCE):
						if (xmlChild.hasAttr(Globals.WAS_KEY_ATTR_NAME)):
							wasKey = xmlChild.getAttrValue(Globals.WAS_KEY_ATTR_NAME)
							configElements.addReferenceId(wasKey, xmlChild)
						else:
							self.LOGGER.info("Reference xmlNode does not have attribute " + Globals.WAS_KEY_ATTR_NAME +
										" - processing node will not continue.")
						#endIf			
					else:
						self._createWASObject(xmlChild, newId)
					#endIf			
				else:
					self.LOGGER.info(xmlChild.getNodeNameFixed() + " type is in list of types to exclude.")
				#endIf
			#endFor
			## update the references
			if (self.updateReferences):
				for wasKey in configElements.referenceIds.keys():
					# update references
					childReferenceNodes = configElements.referenceIds[wasKey]
					self._updateWASReferenceByKey(childReferenceNodes, newId, wasKey, excludedTypes)
				#endFor
			else:
				self.LOGGER.debug("References will not be updated for attributes: " 
							+ str(configElements.referenceIds.keys()) + " on parent " + parentId)		
			#endIf
		#endIf
		self.LOGGER.traceExit(newId)
		return newId
	#endDef

	"""
		Creates the entity in WAS defined by the supplied xmlNode.
		
		Children are not created, use createWASObject to include children.
		
		After the object is created, the objectId is set in the supplied xmlNode as configId
		
		@param parentId: The id of the parent object in WAS
		@param xmlNode: Contains the attributes which define the object.
		@param excludedAttributes: Attributes that exist in the xmlNode 
		                           but should not be created in WAS 
		@raise ConfigWriterException: If the entity cannot be created in WAS 
		@return: The id of the entity in WAS, never None
	"""
	def createEntity(self, xmlNode, parentId, excludedAttributes=[]):
		Performance.start("ConfigWriter.createEntity")
		type = xmlNode.getNodeNameFixed()
		attrs = xmlNode.buildNodeAttrsAsJyString(excludedAttributes)
		newId = None
		if ( xmlNode.hasAttr(Globals.WAS_KEY_ATTR_NAME) ):
			## case where we need a fourth parameter
			wasAttrName = xmlNode.getAttrValue(Globals.WAS_KEY_ATTR_NAME)
			self.LOGGER.debug("AdminConfig.create('"+type+"','"+ parentId+"',"+ attrs +",'"+ wasAttrName+"')")
			newId = AdminConfig.create(type, parentId, attrs, wasAttrName)
		else: 
			self.LOGGER.debug("AdminConfig.create('"+type+"','"+parentId+"',"+ attrs +")")
			newId = AdminConfig.create(type, parentId, attrs)
		#endIf
		if (newId is None or len(newId) == 0):
			msg = ("Unable to create type: " + type + " under parent: " + parentId 
				+ " for xmlNode: " + str(xmlNode))
			self.LOGGER.error(msg)
			raise ConfigWriterException(msg)
		#endIf
		xmlNode.setConfigId(newId)
		print  "Created config object " + type + ": " + newId
		Performance.end("ConfigWriter.createEntity")
		return newId
	#endDef
	
	"""
	##--------------------------------------------------------------------
	## Removes configuration data from WAS
	## This method does not remove "builtin" WAS configuration types
	##
	## @param scopeType: cell/node/cluster/server - used to filter out nodes not in scope
	## @param typeName: The WAS configuraton type to remove, usually given by xmi:id in WAS config files
	## @param scopeId: Optional param used to specify the parent id to start search from
	## 
	##--------------------------------------------------------------------
	"""
	def removeExistingConfig(self, scopeType, typeName, scopeId=None, excludedTypes = []):
		self.LOGGER.traceEnter([scopeType, typeName, scopeId, excludedTypes])
		if (scopeId == None):
			existingConfigIds = AdminConfig.list(typeName).split(newline)
		else:
			existingConfigIds = AdminConfig.list(typeName, scopeId).split(newline)
		#endIf

		for existingId in existingConfigIds:
			## if anything found, is in scope, and not builtin
			if (len(existingId) > 0 and AdminHelper.isInScope(scopeType, existingId) and (existingId.find("builtin") == -1)):
				if (excludedTypes != []):
					exclude = 0
					for excludedType in excludedTypes:
						if (SystemUtils.regexp(excludedType, existingId) == 1):
							exclude = 1
						#endIf
					#endFor
					if (not exclude):
						self.remove(existingId, excludedTypes)
					#endIf
				else:
					self.remove(existingId, excludedTypes)
				#endIf
			#endIf
		#endFor
		self.LOGGER.traceExit()
	#endDef
	
	"""
	##--------------------------------------------------------------------
	## Extends AdminConfig.remove to remove all elements except those 
	## whose config type is in the supplied list of excludedTypes
	##
	## @param existingId:  The WebSphere configuration id to remove
	## @param excludedTypes: A list of types to exclude in the removal
	##--------------------------------------------------------------------
	"""
	def remove(self, existingId, excludedTypes):
		self.LOGGER.traceEnter([existingId, excludedTypes])
		if (len(existingId) > 0):
			configType = self.configReader.getWASType(existingId)
			if (configType not in excludedTypes):
				print  "Removing the existing config object " + existingId
				AdminConfig.remove(existingId)
			else:
				print "Type for id=" + existingId + " found in excludedTypes"
				print "object will not be removed from WebSphere configuration"
			#endif
		else:
			print "Blank existingId supplied, returning without doing anything"
		self.LOGGER.traceExit()
	#endDef
	
	"""
		Extends AdminConfig.modify to update element and their children
		unless  their config type is in the supplied list of excludedTypes
	
		After the object is modified, the objectId is set in the supplied xmlNode as configId

	 	@param id:  The WebSphere configuration id to modify
	 	@param xmlNode: An xml node object that contains the attribute 
	                 name/value pair to update
	 	@param excludedTypes: A list of types to exclude in the removal
	"""
	def modify(self, id, xmlNode, excludedTypes = ['builtin']):
		self.LOGGER.traceEnter([id, xmlNode, excludedTypes])
		self.LOGGER.debug("Updating: " + id)
		configType = xmlNode.getNodeNameFixed()
		if (configType not in excludedTypes):
			AdminConfig.modify(id, xmlNode.buildNodeAttrsAsJyString(excludedTypes))
			xmlNode.setConfigId(id)
			self._updateWASChildren(xmlNode, id, excludedTypes)
		else:
			print "Type for id=" + id + " found in excludedTypes"
			print "object and its children will not be modified in WebSphere configuration"
		#endIf
		self.LOGGER.traceExit()
	#endDef
	
		"""
		Extends AdminConfig.modify to update element and their children
		unless  their config type is in the supplied list of excludedTypes
	
		After the object is modified, the objectId is set in the supplied xmlNode as configId

	 	@param id:  The WebSphere configuration id to modify
	 	@param xmlNode: An xml node object that contains the attribute 
	                 name/value pair to update
	 	@param excludedTypes: A list of types to exclude in the removal
	"""
	def trueModify(self, id, xmlNode, excludedTypes = ['builtin']):
		self.LOGGER.traceEnter([id, xmlNode, excludedTypes])
		self.LOGGER.debug("Updating: " + id)
		configType = xmlNode.getNodeNameFixed()
		if (configType not in excludedTypes):
			AdminConfig.modify(id, xmlNode.buildNodeAttrsAsJyString(excludedTypes))
			xmlNode.setConfigId(id)
			self._updateWASChildren(xmlNode, id, excludedTypes, 1)
		else:
			print "Type for id=" + id + " found in excludedTypes"
			print "object and its children will not be modified in WebSphere configuration"
		#endIf
		self.LOGGER.traceExit()
	#endDef
	
	"""
	##--------------------------------------------------------------------
	## Updates Websphere Configuration by WAS key by first reading the existing config
	## ids, updating those that exist in the xmlNodes array, removing those that don't,
	## and creating any configuration in the xmlNodes array which does not exist as an 
	## existing id
	##
	## @param xmlNodes:  xml configuration from RAFW 
	## @param parentId: the WebSphere id of the parent object
	## @param keyName: the WebSphere key to update elements.  Must be a key of the parent
	##
	## @note: This method should be considered Private 
	##        and not called outside of this class
	##--------------------------------------------------------------------
	"""
	def updateWASObjectByKey(self, xmlNodes, parentId, keyName, excludedTypes):
		Performance.start("ConfigWriter.updateWASObjectByKey")
		self.LOGGER.debug("ByKey: updating config under: " + parentId + " modifying elements under key: " + keyName)
	
		## show existing ids for the key
		existingIds = AdminConfig.showAttribute(parentId, keyName)
		
		## turn list into array
		existingIdList = AdminHelper.extractWASIdsFromList(existingIds)
		## update the existing ids
		self.processChildren(existingIdList, xmlNodes, parentId, excludedTypes)
		Performance.end("ConfigWriter.updateWASObjectByKey")
	#endDef
	
		"""
	##--------------------------------------------------------------------
	## Updates Websphere Configuration by WAS key by first reading the existing config
	## ids, updating those that exist in the xmlNodes array, removing those that don't,
	## and creating any configuration in the xmlNodes array which does not exist as an 
	## existing id
	##
	## @param xmlNodes:  xml configuration from RAFW 
	## @param parentId: the WebSphere id of the parent object
	## @param keyName: the WebSphere key to update elements.  Must be a key of the parent
	##
	## @note: This method should be considered Private 
	##        and not called outside of this class
	##--------------------------------------------------------------------
	"""
	def trueUpdateWASObjectByKey(self, xmlNodes, parentId, keyName, excludedTypes):
		Performance.start("ConfigWriter.updateWASObjectByKey")
		self.LOGGER.debug("ByKey: updating config under: " + parentId + " modifying elements under key: " + keyName)
	
		## show existing ids for the key
		existingIds = AdminConfig.showAttribute(parentId, keyName)
		
		## turn list into array
		existingIdList = AdminHelper.extractWASIdsFromList(existingIds)
		## update the existing ids
		self.processChildrenForTrueUpdate(existingIdList, xmlNodes, parentId, excludedTypes)
		Performance.end("ConfigWriter.updateWASObjectByKey")
	#endDef

	"""
	##--------------------------------------------------------------------
	## Contains the logic used to update existing was configuration based on the
	## incoming xml configuration nodes.
	## Will create anything that is in the configuration and not found in WAS
	##
	## @param existingIds:  The existing websphere configuration ids
	## @param xmlNodes: Incoming xml configuration from RAFW
	## @param parentId: The websphere id of the parent for these configuration changes
	##
	## @note: This method is only called from within this class and in managerServer.py
	##        Its use outside of this class is discouraged
	##--------------------------------------------------------------------
	"""
	def processChildrenForTrueUpdate(self, existingIds, xmlNodes, parentId, excludedTypes):
		self.LOGGER.traceEnter([existingIds, xmlNodes, parentId, excludedTypes])
		## special case, there is 1 node and 1 id and they both reference the same config type
		## update the id with the attributes of the node
		if (len(existingIds) == 1 and len(xmlNodes) == 1):
			xmlNode = xmlNodes[0]
			xmlType = xmlNode.getNodeNameFixed()
			configType = self.configReader.getWASType(existingIds[0])
			self.LOGGER.trace("1 id and 1 xmlNode found.  xmlType=" + str(xmlType) + " and configType=" + str(configType))
			if (xmlType == configType):
				if (xmlNode.hasAttr(Globals.RAFW_TYPE) and xmlNode.getAttrValue(Globals.RAFW_TYPE) == Globals.RAFW_TYPE_REFERENCE):
					## no reference xml nodes should be handled through this code path
					self.LOGGER.error("References will not be updated for xmlNode: " + configType)
				else:
					self.trueModify(existingIds[0], xmlNode, excludedTypes)
				#endIf
				self.LOGGER.traceExit()
				return
			#endIf
		#endIf
		## hash the existing ids in WAS
		## objHash is key-was_id pair of unique ids
		## unidentified is a list of ids which cannot be identified
		## nonUniqueKeys is a list of keys which are not unique
		objHash, unidentified, nonUniqueKeys = self.hashExistingIds(existingIds)
		configsToCreate = []
		for xmlNode in xmlNodes:
			#print "---------xmlChild: " + str(xmlChild)
			isSkippableChild = 0
			if xmlNode.getNodeNameFixed() == "J2EEResourceProperty":
				descContainsBracket = 0
				if xmlNode.hasAttr('description'):
					descContainsBracket = xmlNode.getAttrValue('description').find('[') == 0
				#endIf
				if xmlNode.hasAttr('value'):
					valueContainsBracket = xmlNode.getAttrValue('value').find('[') == 0
				#endIf
				isSkippableChild = valueContainsBracket or descContainsBracket
			#endIf
			if isSkippableChild:
				continue
			#endIf
			key = self.computeXmlKey(xmlNode)
			if (objHash.has_key(key)):
				obj = objHash[key]
				# update matching config element
				self.updatedIds.append(obj)
				if (xmlNode.hasAttr(Globals.RAFW_TYPE) and xmlNode.getAttrValue(Globals.RAFW_TYPE) == Globals.RAFW_TYPE_REFERENCE):
					## no reference xml nodes should be handled through this code path
					self.LOGGER.error("References will not be updated for xmlNode: " + xmlNode)
				else:
					self.trueModify(obj, xmlNode, excludedTypes)
				#endIf
				# remove the updated object id from the hash
				del objHash[key]
			else:
				# create with the parent later
				configsToCreate.append([xmlNode, parentId])
			#endIf
		#endFor	
		## create the configs which did not exist before or could not be matched with existing configuration
		for config in configsToCreate:
			xmlNode = config[0]
			parentId = config[1]
			wasType = xmlNode.getNodeNameFixed()
			print "Creating: " + wasType
			self._createWASObject(xmlNode, parentId, excludedTypes)
		#endFor
		self.LOGGER.traceExit()
	#endDef

	"""
	##--------------------------------------------------------------------
	## Contains the logic used to update existing was configuration based on the
	## incoming xml configuration nodes.
	## Will delete anything not in the xml configuration
	## Will create anything that is in the configuration and not found in WAS
	##
	## @param existingIds:  The existing websphere configuration ids
	## @param xmlNodes: Incoming xml configuration from RAFW
	## @param parentId: The websphere id of the parent for these configuration changes
	##
	## @note: This method is only called from within this class and in managerServer.py
	##        Its use outside of this class is discouraged
	##--------------------------------------------------------------------
	"""
	def processChildren(self, existingIds, xmlNodes, parentId, excludedTypes):
		self.LOGGER.traceEnter([existingIds, xmlNodes, parentId, excludedTypes])
		
		## special case, there is 1 node and 1 id and they both reference the same config type
		## update the id with the attributes of the node
		if (len(existingIds) == 1 and len(xmlNodes) == 1):
			xmlNode = xmlNodes[0]
			xmlType = xmlNode.getNodeNameFixed()
			configType = self.configReader.getWASType(existingIds[0])
			self.LOGGER.trace("1 id and 1 xmlNode found.  xmlType=" + str(xmlType) + " and configType=" + str(configType))
			if (xmlType == configType):
				if (xmlNode.hasAttr(Globals.RAFW_TYPE) and xmlNode.getAttrValue(Globals.RAFW_TYPE) == Globals.RAFW_TYPE_REFERENCE):
					## no reference xml nodes should be handled through this code path
					self.LOGGER.error("References will not be updated for xmlNode: " + configType)
				else:
					self.modify(existingIds[0], xmlNode, excludedTypes)
				#endIf
				self.LOGGER.traceExit()
				return
			#endIf
		#endIf
		
		## hash the existing ids in WAS
		## objHash is key-was_id pair of unique ids
		## unidentified is a list of ids which cannot be identified
		## nonUniqueKeys is a list of keys which are not unique
		objHash, unidentified, nonUniqueKeys = self.hashExistingIds(existingIds)
		
		configsToCreate = []
		
		for xmlNode in xmlNodes:
			key = self.computeXmlKey(xmlNode)
			if (objHash.has_key(key)):
				obj = objHash[key]
				# update matching config element
				self.updatedIds.append(obj)
				if (xmlNode.hasAttr(Globals.RAFW_TYPE) and xmlNode.getAttrValue(Globals.RAFW_TYPE) == Globals.RAFW_TYPE_REFERENCE):
					## no reference xml nodes should be handled through this code path
					self.LOGGER.error("References will not be updated for xmlNode: " + xmlNode)
				else:
					self.modify(obj, xmlNode, excludedTypes)
				#endIf
				# remove the updated object id from the hash
				del objHash[key]
			else:
				# create with the parent later
				configsToCreate.append([xmlNode, parentId])
			#endIf
		#endFor	
		
		## delete all the objects who don't have the name attribute
		## they will be recreated later
		## ids can be referenced in multiple places 
		## - need to make sure we don't delete one that was referenced elsewhere
		for id in unidentified:
			if (len (id) > 0):
				if (id not in self.updatedIds):
					try:
						self.remove(id, excludedTypes)
					except:
						self.LOGGER.error(str(sys.exc_info()[0]))
						if (sys.exc_info()[1] is not None):
							self.LOGGER.error("    Details: " + str(sys.exc_info()[1]))
						print "The following object id no longer exists: " + id
				#endIf
			#endIf
		#endFor
		# remove the extra config elements which were not updated
		for id in objHash.values():
			if (len(id) > 0):
				if (id not in self.updatedIds):
					try:
						self.remove(id, excludedTypes)
					except:
						self.LOGGER.error(str(sys.exc_info()[0]))
						if (sys.exc_info()[1] is not None):
							self.LOGGER.error("    Details: " + str(sys.exc_info()[1]))
						print "The following object id no longer exists: " + id
				#endIf
			#endIf
		#endFor
		## create the configs which did not exist before or could not be matched with existing configuration
		for config in configsToCreate:
			xmlNode = config[0]
			parentId = config[1]
			wasType = xmlNode.getNodeNameFixed()
			print "Creating: " + wasType
			self._createWASObject(xmlNode, parentId, excludedTypes)
		#endFor
		self.LOGGER.traceExit()
	#endDef

	"""
	##--------------------------------------------------------------------
	## Hashes the existing Websphere Ids.
	## @return:  [objHash, unidentified, nonUniqueKeys]
	##           objHash is key-was_id pair of unique ids
	##           unidentified is a list of ids which cannot be identified
	##           nonUniqueKeys is a list of keys which are not unique
	##
	## @note: This method should be considered Private 
	##        and not called outside of this class
	##--------------------------------------------------------------------
	"""
	def hashExistingIds(self, existingIds):
		self.LOGGER.traceEnter([existingIds])
		
		# if more than 1, update using name as key
		# best NOT to re-create - must compare based on name and update or create
		
		## hash of name attr to obj id
		objHash = {}
		## list of ids which cannot be uniquely identified
		unidentified = []
		nonUniqueKeys = []
	
		## build a hash of existing configuration objects to be later updated or removed
		for obj in existingIds:
			if (len(obj) > 0):
				wasType = self.configReader.getWASType(obj)
				uniqueAttrs = self.configTypeKeys.getUniqueAttrs(wasType)
				key = self.computeWasKey(obj)
				## determine if the key is being used by another
				if (objHash.has_key(key)):
					self.LOGGER.info(str(uniqueAttrs) + " returns a value that is not unique: " 
									+ key + " for config type: " + wasType)
					nonUniqueKeys.append(key)
					## key already found, id cannot be identified
					unidentified.append(objHash[key])
					unidentified.append(obj)
					del objHash[key]
				elif (key in nonUniqueKeys):
					unidentified.append(obj)
				else:
					## key can be identified
					objHash[key] = obj	
				#endIf
			#endIf
		#endFor
		self.LOGGER.traceExit([objHash, unidentified, nonUniqueKeys])
		return [objHash, unidentified, nonUniqueKeys]
	#endDef


	def computeXmlKey(self, xmlNode):
		"""
		Computes a key that is used to identify this object in the XML Configuration.  A similar
		key will be used to identify the object's ID in the WebSphere configuration 
		
		This method can be overridden to compute keys in a specific way for config types.
		
		@param xmlNode: The xmlNode as read from the xml file, can contain children.
		@return: A key used to compare this XMLNode against a WebSphere object 
		"""
		self.LOGGER.traceEnter(xmlNode)
		wasType = xmlNode.getNodeNameFixed()
		uniqueAttrs = self.configTypeKeys.getUniqueAttrs(wasType)
		configName = ""
		if (xmlNode.hasAllAttrs(uniqueAttrs)):
			for uniqueAttr in uniqueAttrs:
				if (len(configName) > 0):
					configName = configName + "."
				configName = configName + xmlNode.getAttrValue(uniqueAttr)
			#endFor
		#endIf
		key = wasType + "." + configName
		self.LOGGER.traceExit(key)
		return key
	#endDef
	
	def computeWasKey(self, obj):
		"""
		Computes a key that is used to identify this object in WebSphere.  A similar
		key will be used to identify this object's configuration in the XML Node
		
		This method can be overridden to compute keys in a specific way for config types.
		
		@param obj: The WebSphere Object ID
		@return: A key used to hash this object id.
		"""
		self.LOGGER.traceEnter(obj)
		if (obj == None or len(obj) == 0):
			raise ConfigWriterException("Configuration id is none while finding WAS attributes to use to identify this object")
		configName = ""
		wasType = self.configReader.getWASType(obj)
		uniqueAttrs = self.configTypeKeys.getUniqueAttrs(wasType)
		if (self.configReader.hasAllAttrs(obj, uniqueAttrs)):
			for uniqueAttr in uniqueAttrs:
				if (len(configName) > 0):
					configName = configName + "."
				#endIf
				self.LOGGER.trace("reading value of '" + uniqueAttr + "' on object:'" + obj + "'")
				attrValue = AdminConfig.showAttribute(obj, uniqueAttr)
				if (attrValue != None):
					configName = configName + attrValue
			#endFor
		else:
			self.LOGGER.warn(str(uniqueAttrs) + " attributes not available on AdminConfig objId=" + obj)
			configName = ""
		#endIf
		key = wasType + "." + configName
		self.LOGGER.traceExit(key)
		return key
	#endDef
	
	"""
		Searches for matching objects in WAS and sets the first 
		matching id to the attribute given by wasKey
		in the supplied parentId
		
		@param xmlNodes:  A collection of references to locate and set into wasKey.
		                  If more than 1 node is supplied, the ids will be space 
		                  delimited and stored in WAS under attribute wasKey in
		                  the parent object given by parentId
		
		@raise ConfigWriterException: If no object id is found in WebSphere 
		                              which matches the search attributes
	"""
	def _updateWASReferenceByKey(self, xmlNodes, parentId, wasKey, excludedTypes, scopeId):
		if (xmlNodes is None):
			msg = "xmlNodes is None.  Unable to locate reference object IDs with invalid search criteria."
			self.LOGGER.error(msg)
			raise ConfigWriterException(msg)
		if (parentId is None or len(parentId) == 0):
			msg = ("parentId is None or an empty string.  Unable to set reference object IDs in the "
				+ "WebSphere configuration without a parent configuration ID")
			self.LOGGER.error(msg)
			raise ConfigWriterException(msg)
		if (wasKey is None or len(wasKey) == 0):
			msg = ("wasKey is None or an empty string in supplied configuration.  "
				+ "Unable to set reference object IDs on the WebSphere configuration object with id '"
				+ str(parentId) + "'")
			self.LOGGER.error(msg)
			raise ConfigWriterException(msg)
	
		referenceIds = ""
		for xmlNode in xmlNodes:	
			wasType = xmlNode.getNodeNameFixed() 
			if (wasType not in excludedTypes):
				referenceId = self.getReferenceId(xmlNode, scopeId)
				if (referenceId is not None):
					if (len(referenceIds) == 0):
						referenceIds = referenceId
					else:
						referenceIds = referenceIds + " " + referenceId
					#endIf
				else:
					if (self.handleReferenceSearchError(xmlNode, wasKey, parentId)):
						return
				#endIf
			else:
				self.LOGGER.info(wasType + " is on the excludedTypes list.  " 
								+ "Reference Object ids will not be updated for: " + str(xmlNode))
			#endIf
		#endFor
		if (len(referenceIds) > 0):
			self.LOGGER.info("Setting reference objectId for attribute " + wasKey 
						+ " in parent object: " + parentId + " to value: " + referenceIds)
			## this must be a string as the second argument, or any unicode characters will result in failure
			## attributes in this list must be separated by a space ( ) 
			AdminConfig.modify(parentId, "[['" + wasKey + "' '" + referenceIds + "']]")
		#endIf
	#endDef
	
	def handleReferenceSearchError(self, xmlNode, wasKey, parentId):
		"""
			Indicates that an error was encountered while searching for reference ids
			
			This method can be overridden.
				
			Typically this method throws an exception to stop processing and report back to the user.
			However, if an exception will not be thrown, this method can:
			return false if the xmlNode should be ignored and processing should continue on to the next xmlNode
			return true if all xmlNodes related to this wasKey should be ignored
		"""
		msg = ("The search for a reference object returned invalid results using search attributes: " 
			+ str(xmlNode) + "  References can not be updated for attribute: " 
			+ wasKey + " on parent " + parentId)
		self.LOGGER.error(msg)
		raise ConfigWriterException(msg)
	#endDef
	
	def getReferenceId(self, xmlNode, scopeId = None):
		"""
		Finds the object id in WebSphere configuration which matches the supplied xmlNode.
		Returns the object id or None if none can be found or if more than 1 match is found.
		Uses the MissingReferenceIdHandler to create missing references if none can be found.
		This method can be overridden in a subclass when necessary
		"""
		self.LOGGER.traceEnter(str(xmlNode))
		referenceId = None
		if 'parentMgmtScopeName' in globals().keys():
			discoveredIds = self.configReader.findObjectIds(xmlNode, scopeId, parentMgmtScopeName)
		else:
			discoveredIds = self.configReader.findObjectIds(xmlNode, scopeId)
		#endIf
		if (len(discoveredIds) == 0 and self.missingReferenceIdHander is not None):
			referenceId = self.missingReferenceIdHander.handleMissingReferenceId(xmlNode)
		elif (len(discoveredIds) > 1):
			## not an error, but will help with debugging
			self.LOGGER.debug("Found " + str(len(discoveredIds)) + " matches for xmlNode: " + str(xmlNode) + " in WebSphere configuration.")
			self.LOGGER.debug("Found the following matches: " + str(discoveredIds))
			if (self.initialScopeId is not None):
				self.LOGGER.debug("Trying again with a smaller scope: " + str(self.initialScopeId))
				## try again with tighter scope
				if 'parentMgmtScopeName' in globals().keys():
					discoveredIds = self.configReader.findObjectIds(xmlNode, self.initialScopeId, parentMgmtScopeName)
				else:
					discoveredIds = self.configReader.findObjectIds(xmlNode, self.initialScopeId)
				#endIf
				if (len(discoveredIds) == 1):
					referenceId = discoveredIds[0]
				else:
					if (self.missingReferenceIdHander is not None):
						self.LOGGER.debug("Using ReferenceIdHandler to identify the reference id")
						referenceId = self.missingReferenceIdHander.findReferenceId(xmlNode)
					else:
						## handler is not set to handle the problem, tell the user so they can troubleshoot
						self.LOGGER.info("Found " + str(len(discoveredIds)) + " matches for xmlNode: " 
										+ str(xmlNode) + " in scope: " + str(self.initialScopeId))
						self.LOGGER.info("Found the following matches: " + str(discoveredIds))
						referenceId = None
					#endIf
				#endIf
			else:
				## now its an error, tell user what is up so that they can troubleshoot
				self.LOGGER.debug("No initialScopeId has been set on this object.")
				self.LOGGER.info("Found " + str(len(discoveredIds)) + " matches for xmlNode: " + str(xmlNode) + " in WebSphere configuration.")
				self.LOGGER.info("Found the following matches: " + str(discoveredIds))
				referenceId = None
			#endIf
		elif (len(discoveredIds) == 1):
			referenceId = discoveredIds[0]
		#endIf
		self.LOGGER.traceExit(referenceId)
		return referenceId
	#endDef
	
	"""
		Updates the WAS configuration data using data in the supplied xmlNodes
		
		As all objects are updated, the objectId is set in the xmlNode as configId

		@param xmlNodes: Array of xmlNodes used as data to update
		@param parentId: The id of the parent in the configuration to be updated
		@param type: WAS config type to be updated, usually given by xmi:id in WAS 
	             configuration files
	"""
	def updateWASObject(self, xmlNodes, parentId, type, excludedTypes=[]):
	
		Performance.start("ConfigWriter.updateWASObject")
		self.LOGGER.info("updating config under: " + str(parentId) + " modifying type: " + str(type))
		self.initialScopeId = parentId
	
		## As this method is the entry to the update functionality it resets the list of ids
		## that have been processed.
		self.updatedIds = []
		self.updatedIds.append(parentId)
		objs = AdminConfig.list(type,parentId)
		if (len(objs) != 0):
			objs = objs.split(newline)
		self.processChildren(objs, xmlNodes, parentId, excludedTypes)
		Performance.end("ConfigWriter.updateWASObject")
	
	#endDef
	
	"""
		Updates the WAS configuration data using data in the supplied xmlNodes
		
		As all objects are updated, the objectId is set in the xmlNode as configId

		@param xmlNodes: Array of xmlNodes used as data to update
		@param parentId: The id of the parent in the configuration to be updated
		@param type: WAS config type to be updated, usually given by xmi:id in WAS 
	             configuration files
	"""
	def trueUpdateWASObject(self, xmlNodes, parentId, type, excludedTypes=[]):
	
		Performance.start("ConfigWriter.updateWASObject")
		self.LOGGER.info("updating config under: " + str(parentId) + " modifying type: " + str(type))
		self.initialScopeId = parentId
	
		## As this method is the entry to the update functionality it resets the list of ids
		## that have been processed.
		self.updatedIds = []
		self.updatedIds.append(parentId)
		objs = AdminConfig.list(type,parentId)
		if (len(objs) != 0):
			objs = objs.split(newline)
		self.processChildrenForTrueUpdate(objs, xmlNodes, parentId, excludedTypes)
		Performance.end("ConfigWriter.updateWASObject")
	
	#endDef
	
	"""
	##--------------------------------------------------------------------
	## Processes children of the xmlNode to update.
	## Elements of the same type are updated together to allow for 
	## proper differences, (elements added, updated, and removed based on input)
	##
	## @param xmlNode: The node containing children to create/update in the 
	##                 WAS runtime configuration
	## @param parentId: The id of the WAS configuration parent under 
	##                  which data should be updated
	##
	##--------------------------------------------------------------------
	"""
	def updateWASChildren(self, xmlNode, parentId, excludedTypes):
		self.LOGGER.traceEnter([xmlNode, parentId, excludedTypes])
		self.initialScopeId = parentId
		self._updateWASChildren(xmlNode, parentId, excludedTypes)
		self.LOGGER.traceExit()
	#endDef
	
	"""
		This method is called recursively by other methods in this class
	"""
	def _updateWASChildren(self, xmlNode, parentId, excludedTypes, trueModify = 0):
		self.LOGGER.traceEnter([xmlNode, parentId, excludedTypes])
		configElements = ConfigElements()
		## Seed the configElements list with attribute names that contain children
		## so that if the child is not defined in the xmlNode, it will be removed from
		## the WebSphere runtime configuration
		childrenKeys = self.configReader.getChildrenKeys(parentId, excludedTypes)
		for wasKey in childrenKeys:
			configElements.initializeLowPriorityId(wasKey)
		#endIf
			
		for child in xmlNode.getChildrenArray():
			if (child.hasAttr(Globals.WAS_KEY_ATTR_NAME)):
				wasKey = child.getAttrValue(Globals.WAS_KEY_ATTR_NAME)
				if (child.hasAttr(Globals.RAFW_TYPE) and child.getAttrValue(Globals.RAFW_TYPE) == Globals.RAFW_TYPE_REFERENCE):
					## reference node
					configElements.addReferenceId(wasKey, child)
				else:
					## regular node
					configElements.addLowPriorityId(wasKey, child)
			else:
				print "WARN: xmlNode does not have attribute " + Globals.WAS_KEY_ATTR_NAME
				print " - processing node will not continue."
			#endIf
		#endFor
		## update children
		for wasKey in configElements.lowPriorityIds.keys():
			# pass in similar children
			xmlNodes = configElements.lowPriorityIds[wasKey]
			if trueModify:
				self.trueUpdateWASObjectByKey(xmlNodes, parentId, wasKey, excludedTypes)
			else:
				self.updateWASObjectByKey(xmlNodes, parentId, wasKey, excludedTypes)
		#endFor
		## update references
		if (self.updateReferences):
			for wasKey in configElements.referenceIds.keys():
				# update references
				xmlNodes = configElements.referenceIds[wasKey]
				self._updateWASReferenceByKey(xmlNodes, parentId, wasKey, excludedTypes)
			#endFor
		else:
			self.LOGGER.debug("References will not be updated for attributes: " 
							+ str(configElements.referenceIds.keys()) + " on parent " + parentId)		
		#endIf
		self.LOGGER.traceExit()
	#endDef

	"""
		Finds nodes in the supplied xmlNodes list that are of type RAFW_TYPE_REFERENCE
		and uses those nodes to update references on their parents.
		
		The supplied scopeId is used to locate parents on which to set references.
		
		Types in the excludedTypes array will be ignored when finding and updating
		reference ids.
	"""
	def updateWASReferenceAttributes(self, xmlNodes, scopeId, excludedTypes = []):
		self.LOGGER.traceEnter([xmlNodes, scopeId, excludedTypes])
		self.initialScopeId = scopeId
		for xmlNode in xmlNodes:
			wasType = xmlNode.getNodeNameFixed() 
			if (wasType not in excludedTypes):
				parentId = xmlNode.getConfigId()
				if (parentId is None):
					parentIds = self.configReader.findObjectIds(xmlNode, scopeId)
					if (len(parentIds) == 1):
						parentId = parentIds[0]
					else:
						self.LOGGER.error("Unable to find object ID for " 
							+ str(xmlNode) + " in scope: " + str(scopeId))
						self.LOGGER.info("Found these object ids: " + str(parentIds))
						raise ConfigWriterException("Unable to determine object ID while updating references for " + str(xmlNode))
					#endIf
				#endIf
				xmlChildren = xmlNode.getChildrenArray()
				global parentMgmtScopeName
				parentMgmtScopeName = None
				if xmlNode.getNodeNameFixed() == "SSLConfig":
					xmlAttrs = self.configReader.show(xmlNode.getConfigId())
					parentMgmtScopeName = self.configReader.show(xmlAttrs['managementScope'])['scopeName']
				#endIf
				self._updateReferenceAttributes(xmlChildren, parentId, excludedTypes, scopeId)
			else:
				self.LOGGER.debug("Nodes of type: " + str(wasType) + " have been excluded and will not be processed.")
			#endIf
		#endFor
		self.LOGGER.traceExit()
	#emdDef

	"""
		If the supplied xmlNode is a RAFW_TYPE_REFERENCE, it is used to locate the WAS
		configuration id which is then set for the attribute given by WAS_KEY on the 
		parent configuration object.
		
		Types in the excludedTypes list are excluded
	"""
	def _updateReferenceAttributes(self, xmlNodes, parentId, excludedTypes, scopeId= None):
		self.LOGGER.traceEnter([xmlNodes, parentId, excludedTypes])
		configElements = ConfigElements()
		for xmlNode in xmlNodes:
			wasType = xmlNode.getNodeNameFixed() 
			if (wasType not in excludedTypes):
				if (xmlNode.hasAttr(Globals.RAFW_TYPE) and xmlNode.getAttrValue(Globals.RAFW_TYPE) == Globals.RAFW_TYPE_REFERENCE):
					parentAttribute = xmlNode.getAttrValue(Globals.WAS_KEY_ATTR_NAME)				
					if (parentAttribute is None or len(parentAttribute) == 0):
						msg = (Globals.WAS_KEY_ATTR_NAME + " is undefined in the supplied XML configuration '"
							+ str(xmlNode) + "' "
							+ "Unable to set reference object IDs on the WebSphere configuration object with id '"
							+ str(parentId) + "'")
						self.LOGGER.error(msg)
						raise ConfigWriterException(msg)
					#endIf
					configElements.addReferenceId(parentAttribute, xmlNode)
				#endIf
				xmlChildren = xmlNode.getChildrenArray()
				if (len(xmlChildren) > 0):
					nextParentId = xmlNode.getConfigId()
					if (nextParentId is None):
						nextParentId = self.configReader.getObjectId(xmlNode, parentId)
					#endIf
					if (nextParentId is not None):
						self._updateReferenceAttributes(xmlChildren, nextParentId, excludedTypes, scopeId)
					else:
						self.LOGGER.error("Unable to find Object ID for " 
								+ str(xmlNode) + " in scope: " + str(parentId))
						raise ConfigWriterException("Error while updating references for " + str(xmlNode))
					#endIf
				#endIf
			else:
				self.LOGGER.debug("Nodes of type: " + str(wasType) + " have been excluded and will not be processed.")
			#endIf
		#endFor
		
		if (len(configElements.referenceIds.keys()) > 0):
			for wasKey in configElements.referenceIds.keys():
				# update references
				xmlNodes = configElements.referenceIds[wasKey]
				self._updateWASReferenceByKey(xmlNodes, parentId, wasKey, excludedTypes, scopeId)
			#endFor
		#endIf
		self.LOGGER.traceExit()
	#endDef
#endClass

class MissingReferenceIdHandler:
	"""
		Defines a handler which is used to resolve cases where a 
		reference id cannot be found.
	
		When the reference id is not found, the method:
		handleMissingReferenceId(xmlNode) is called.  This method can return the 
		object id that should be used for the reference or return None to indicate 
		that the default behavior should be used.
		
		@see: ConfigWriter._updateWASReferenceByKey()
	"""
	
	def handleMissingReferenceId(self, xmlNode):
		"""
		Used to create referenced objects when the ConfigWriter implementation finds 0 matches
		
		Default implementation always returns None
		"""
		return None
	#endDef
	
	def findReferenceId(self, xmlNode):
		"""
		Used to identify reference ids when more than 1 is found by the ConfigWriter	
		
		Default implementation always returns None
		"""
		return None
	#endDef
#endClass
	
"""
	Thrown when configuration data cannot be written to WAS either
	because the data is invalid or an error occurred while writing 
	the data to WebSphere
"""	
class ConfigWriterException:
	
	def __init__(self, message):
		self.message = message
	#endDef
	
	def __str__(self):
		return str(self.message)
	#endDef
	
#endClass

	
