'''
	Licensed Materials - Property of IBM Corp. 
IBM Rational Automation Framework 
(c) Copyright IBM Corporation 2003, 2012. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 


	Reads properties from an existing cell which are used to generate an
	Environment Cell Definition from an existing WebSphere environment.

'''
from AdminHelper import AdminHelper
from ConfigReader import ConfigReader

from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager

from SystemUtils import SystemUtils
from java.io import File
from java.util import ArrayList

from existingcell.Constants import DR_DEFAULT_VALUE
from existingcell.Constants import NEWLINE
from existingcell.AbstractCellProcessor import AbstractCellProcessor
from existingcell.Node import Node
from existingcell.Server import Server

import AdminControl
import AdminConfig
import AdminTask

import Globals
import java
import sys

'''
	Processor that gathers topology information from a runtime configuration.
	Execution order is: cell, node with servers within node, and clusters.
	This is an abstract superclass. CellDefReader is the owning reader, might be 'None' for testing.
'''
class BPMCellProcessor(AbstractCellProcessor):

	def __init__(self, cellDefReader):
		AbstractCellProcessor.__init__(self, cellDefReader)
		self.LOGGER = _Logger("BPMCellProcessor", MessageManager.RB_WEBSPHERE_BPM)
		self.LOGGER_WAS = _Logger("BPMCellProcessor.WAS", MessageManager.RB_WEBSPHERE_WAS)
		props = {}
		self.environmentDict = props
		self.parent = cellDefReader
		
		self.clusterNum = 0
		self.clusters = ""
		self.managedNodeNum = 0
		self.nodes = []
		self.discoveredNodes = {}
		self.discoveredServers = {}

		self.findProductInfo()
		self.cellType = self.lookup('CELL_TYPE')
	#endDef

	'''
		Called by CellDefinitionReader when a cell is discovered.
	'''
	def cellDiscovered(self, cellId):
		pass

	'''
		Called by CellDefinitionReader when a node is discovered in the cell configuration
	'''
	def nodeDiscovered(self, nodeId):
		self.LOGGER.traceEnter()
		nodeConfig = Node(nodeId)
		self.discoveredNodes[nodeId] = nodeConfig
		self.LOGGER_WAS.log("CRWWA0139I", nodeConfig.getNodeName())
		self.LOGGER.traceExit()
	#endDef

	'''
		Called by CellDefinitionReader when a server is discovered in the node configuration
	'''
	def serverDiscovered(self, nodeId, serverId):
		self.LOGGER.traceEnter()
		node = self.discoveredNodes[nodeId]
		server = Server(serverId, self.platform)
		server.identifyServer(node, self.managedNodeNum)
		if (server.getServerType() == Server.NODE_AGENT):
			self.managedNodeNum += 1
			self.nodes.append(node.getNodeName())
		#endIf
		self.discoveredServers[server.getServerName()] = server
		self.LOGGER.traceExit()
	#endDef

	'''
		Called by CellDefinitionReader when a cluster is discovered in the cell configuration
	'''
	def clusterDiscovered(self, clusterId):
		self.LOGGER.traceEnter()
		clusterName = AdminConfig.showAttribute(clusterId, 'name')
		self.clusters = self.clusters + clusterName + ","
		self.clusterNum +=1
		
		self.LOGGER_WAS.log("CRWWA0143I", clusterName)
		
		self.add("CLUSTER" + str(self.clusterNum), clusterName)
		self.add("CLUSTER" + str(self.clusterNum) + "_CLUSTER_NAME", clusterName)
		self.add("CLUSTER" + str(self.clusterNum) + "_PREFIX", DR_DEFAULT_VALUE)
		self.add("CLUSTER" + str(self.clusterNum) + "_TRANSPORT_NODE_INCREMENTOR", DR_DEFAULT_VALUE)
		self.add("CLUSTER" + str(self.clusterNum) + "_TRANSPORT_STARTING_POINT", DR_DEFAULT_VALUE)
		serverNodeMapping = {}
		
		for clusterMember in AdminConfig.list('ClusterMember', clusterId).split(NEWLINE):
			if (len(clusterMember) > 0):
				serverName = AdminConfig.showAttribute(clusterMember, 'memberName')
				nodeName = AdminConfig.showAttribute(clusterMember, 'nodeName')
				self.LOGGER_WAS.log("CRWWA0144I", [serverName, nodeName])
				if (not serverNodeMapping.has_key(nodeName)):
					serverNodeMapping[nodeName] = ArrayList()
				#endIf
				serverNodeMapping[nodeName].add(serverName)
			#endIf
		#endFor

		nodes = ""
		maxPerNode=0
		for nodeName in serverNodeMapping.keys():
			nodes = nodes + nodeName + ","
			if (serverNodeMapping[nodeName].size() > maxPerNode):
				maxPerNode = serverNodeMapping[nodeName].size()
			#endIf
		#endFor
		self.add("CLUSTER" + str(self.clusterNum) + "_NODES", (nodes[0:len(nodes)-1]))
		self.add("CLUSTER" + str(self.clusterNum) + "_PERNODE", str(maxPerNode))
		self.add("CLUSTER" + str(self.clusterNum) + "_CLUSTER_TYPE", self.lookup('CELL_TYPE'))
		self.LOGGER.traceExit()
	#endDef

	'''
		Finds the product name and version.
	'''
	def findProductInfo(self):
		node = AdminControl.getNode()
		product = AdminTask.getMetadataProperty ('[-nodeName ' + node + ' -propertyName com.ibm.websphere.esbProductShortName]')
		if (product == None or product == ''):
			product = AdminTask.getMetadataProperty ('[-nodeName ' + node + ' -propertyName com.ibm.websphere.wbiProductShortName]')
		#endIf
		
		version = AdminTask.getMetadataProperty ('[-nodeName ' + node + ' -propertyName com.ibm.websphere.esbProductVersion]')
		if (version == None or version == ''):
			version = AdminTask.getMetadataProperty ('[-nodeName ' + node + ' -propertyName com.ibm.websphere.wbiProductVersion]')
		#endIf
		
		if (product == "WPS" or product == "WESB"):
			if version[0:3] == "6.2":
				self.add(product + '_VERSION', '62')
				self.add('CELL_TYPE', product)
			#endif
		#endif        
		self.LOGGER.log("CRWBP0104I", [product, version])
		
	#endDef

	"""
		Retrieves the needed properties related to a node which contains an 
		application server.
		
		Properties are stored in self.environmentDict
	"""

	def _processAppServerNode(self, node):
		self.LOGGER.traceEnter()
		nodeVars = node.getNodeVariables()
		
		self._setNodeProperty("_BASE_STARTING_POINT", DR_DEFAULT_VALUE)
		self._setNodeProperty("_HOST_NAME", AdminConfig.showAttribute(node.getConfigId(), "hostName"))
		nodeName = node.getNodeName()
		self._setNodeProperty("_NODE_NAME", nodeName)
		self._setNodeProperty("_PROFILE_NAME", node.getProfileName())
		
		if (nodeVars.has_key("WAS_INSTALL_ROOT")):
			self._setNodeProperty("_WAS_HOME", nodeVars['WAS_INSTALL_ROOT'])
		else:
			raise "VariableSubstitutionEntry with name='WAS_INSTALL_ROOT' is not defined in the WebSphere configuration."
		#endIf
		if (nodeVars.has_key("USER_INSTALL_ROOT")):
			self._setNodeProperty("_WAS_PROF_HOME", nodeVars['USER_INSTALL_ROOT'])
		else:
			self.add("ND_WAS_PROF_HOME", nodeVars['WAS_INSTALL_ROOT'] + "/profiles/" + node.getProfileName())
		#endIf
		
		self._setNodeProperty("_NODE_TYPE", self.lookup('CELL_TYPE'))
		
		if (self.platform == "zos"):
			self._setNodeProperty("_OS", "ZOS")
		else:
			self._setNodeProperty("_OS", "Non-ZOS")
		#endIf
		self.LOGGER.traceExit()
	#endDef
	
	"""
		Called by CellDefinitionReader after all topology objects have been discovered
		to gather the variables which have been extracted from the configuration
		
		These variables will be added to the env_cell.properties file.
	"""	
	def getVariables(self):
		self.LOGGER.traceEnter()
		numberOfNodes = self.managedNodeNum
		wasVersion = ""
		
		## wasFullVersion is in the form 7.0.0.39
		wasFullVersion = ""
		for node in self.discoveredNodes.values():
			if (node.nodeContainsAppServers):
				self.managedNodeNum = node.managedNodeNum
				self._processAppServerNode(node)
				self._setNodeProperty("_SERVERS_ON_NODE", node.getServersOnNodeStr())
				self._setNodeProperty("_STANDALONE_SERVERS_ON_NODE", node.getStandaloneServersOnNodeStr())
				self._setNodeProperty("_NUMBER_OF_SERVERS", str(node.getNumServers()))
			#endIf
			## add all server properties
			node.setServerProperties(self.environmentDict)
			## all nodes should have the same version
			wasVersion = node.getNodeVersion()
			nodeFullVersion = node.getNodeFullVersion()
			self._setNodeProperty("_WAS_FULL_VERSION", str(nodeFullVersion))
			## take the largest version number in the cell 
			## (this will fail for things like 7.0.0.9 vs 7.0.0.19, but it rarely will happen)
			if (nodeFullVersion > wasFullVersion):
				wasFullVersion = nodeFullVersion
		#endFor
		self.add(self.cellType + "_HOME", self.lookup("ND_WAS_HOME"))
		if (self.cellType == "WPS"):
			self.add("WPS_BPC_PROFILE_NAME", self.lookup("ND_PROFILE_NAME"))
		self.add("WAS_VERSION", wasVersion)
		self.add("WAS_FULL_VERSION", str(wasFullVersion))
		self.add("NODES", ",".join(self.nodes))
		self.add("NUMBER_OF_NODES", numberOfNodes)
		self.add("IS_BASE", "no")
		
		self.add("NUMBER_OF_CLUSTERS", str(self.clusterNum))
		self.add("CLUSTERS", (self.clusters[0:len(self.clusters)-1]))
		self.add("ENABLE_SECURITY_AT_CREATION", DR_DEFAULT_VALUE)
		self.LOGGER.traceExit()
		return self.environmentDict
	#endDef
	
	"""
		Sets a property in the environment dictionary, also used by sub classes
	"""    
	def _setNodeProperty(self, keySuffix, value):
		self.add("NODE" + str(self.managedNodeNum) + keySuffix, value)
	#endDef
	
	'''
		Tries to lookup local variables first (in environmentDict). then parent vars.
	'''
	def lookup(self, key):
		value = self.environmentDict[key]
		if(value != None):
			return value
		if(self.parent != None):
			return self.parent.environmentDict[key]
		return None
	#endDef

	def add(self, key, value):
		self.environmentDict[key] = value
	#endDef

#endClass
