'''
Licensed Materials - Property of IBM Corp. 
IBM Rational Automation Framework 
(c) Copyright IBM Corporation 2003, 2012. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 


@author: Alden Taylor
'''

from existingcell.AbstractCellProcessor import AbstractCellProcessor
from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager
from ConfigReader import ConfigReader
from existingcell.Constants import DR_DEFAULT_VALUE
from existingcell.Constants import NEWLINE
from Node import Node

import AdminConfig

"""
	Processor that gathers IHS topology information from a runtime configuration
"""
class IhsCellProcessor(AbstractCellProcessor):

	def __init__(self, cellDefinitionReader):
		AbstractCellProcessor.__init__(self, cellDefinitionReader)
		self.LOGGER = _Logger("IhsCellProcessor", MessageManager.RB_WEBSPHERE_WAS)
		self.configReader = ConfigReader()
		self.environmentDict = {}
		self.ihsNodeNum = 0
		self.ihsNodes = []
		self.discoveredNodes = {}
		self.platform = ""
	#endDef
	
	"""
		Called by CellDefinitionReader when a cell is discovered. 
		
		This is a No-Op because we do not require any additional information from 
		the cell level for IHS
	"""	
	def cellDiscovered(self, cellId):
		pass
	#endDef
	
	"""
		Called by CellDefinitionReader when a node is discovered in the cell configuration
	"""	
	def nodeDiscovered(self, nodeId):
		nodeConfig = Node(nodeId, self.isAdminAgentCell)
		nodeConfig.setDataType(Node.IHS_NODE)
		self.discoveredNodes[nodeId] = nodeConfig
	#endDef
	
	"""
		Called by CellDefinitionReader when a server is discovered in the node configuration
	"""	
	def serverDiscovered(self, nodeId, serverId):
		#if(self.isAdminAgentCell=="true"):
			#serverType = "WEB_SERVER"
		#else:
		serverType = AdminConfig.showAttribute(serverId, 'serverType')

		if (serverType == 'WEB_SERVER'):
			node = self.discoveredNodes[nodeId]
			self.LOGGER.log("CRWWA0145I", node.getNodeName());
			self._processWebServers(nodeId, serverId)
		#endIf
	#endDef
	
	"""
		Called by CellDefinitionReader when a cluster is discovered in the cell configuration
		
		This is a no-op because there are no IHS clusters
	"""	
	def clusterDiscovered(self, nodeId):
		pass
	#endDef
	
	"""
		Retrieves the needed properties related to the IHS servers in a managed cell
		
		Properties are stored in self.environmentDict
	"""
	def _processWebServers(self, nodeId, serverId):
		self.LOGGER.traceEnter()	
		self.ihsNodeNum += 1
		serverName = AdminConfig.showAttribute(serverId, 'name')
		
		for adminServerAuth in AdminConfig.list( "AdminServerAuthentication", serverId ).split( NEWLINE ):
			if (len(adminServerAuth) > 0):
				adsDict = self.configReader.show(adminServerAuth)
				if (adsDict.has_key("userid") and adsDict["userid"] == ""):
					self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_CREATE_ADMIN_AUTH"] = "false"
				else:
					self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_CREATE_ADMIN_AUTH"] = "true"
					if (adsDict.has_key("userid")):
						self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_AUTH_USERNAME"] = adsDict["userid"]
					else:
						self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_AUTH_USERNAME"] = DR_DEFAULT_VALUE
						self.LOGGER.warn("userid cannot be read for server: " + serverName + ", defaulting to:" + DR_DEFAULT_VALUE)
					#endIf
					if (adsDict.has_key("password")):
						self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_AUTH_PASSWORD"] = adsDict["password"]
					else:
						self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_AUTH_PASSWORD"] = DR_DEFAULT_VALUE
						self.LOGGER.warn("password cannot be read for server: " + serverName + ", defaulting to:" + DR_DEFAULT_VALUE)
					#endIf
				#endIf
			#endIf
		#endFor
		
		for webServer in AdminConfig.list( "WebServer", serverId ).split( NEWLINE ):
			if (len(webServer) > 0):
				adsDict = self.configReader.show(webServer)
				ihsHome = adsDict["webserverInstallRoot"]
				## fix the path on ihs home on windows
				ihsHome = ihsHome.replace("\\" , "/")
				self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_IHS_HOME"] = ihsHome
				
				for pluginProps in AdminConfig.list( "PluginProperties", webServer ).split( NEWLINE ):
					if (len(pluginProps) > 0):
						pluginDict = self.configReader.show(pluginProps)
						self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_PLUGIN_HOME"] = pluginDict["PluginInstallRoot"].replace("\\" , "/")
				#endFor
			#endIf
		#endFor
		
		node = self.discoveredNodes[nodeId] #Node(nodeId)
		portsByServer = node.getPorts()
		ports = portsByServer[serverName]
		if (ports.has_key("WEBSERVER_ADMIN_ADDRESS")):
			self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_IHS_ADMIN_PORT"] = ports["WEBSERVER_ADMIN_ADDRESS"]
		else:
			self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_IHS_ADMIN_PORT"] = "8008"
		#endIf
		if (ports.has_key("WEBSERVER_ADDRESS")):
			self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_HTTP_PORT"] = ports["WEBSERVER_ADDRESS"]
		else:
			self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_HTTP_PORT"] = "80"
		#endIf
	
		self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_HOST_NAME"] = AdminConfig.showAttribute(nodeId, "hostName")
		
		nodeName = node.getNodeName()
		computedNodeName = "ihs_" + nodeName + "_" + serverName
		self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_NODE_NAME"] = computedNodeName
		self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_PROFILE_NAME"] = nodeName
		
		self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_WEB_SERVER_DEF_NAME"] = serverName
		if (self.platform == "zos"):
			self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_OS"] = "ZOS"
		#endIf
		
		self.ihsNodes.append(computedNodeName)
		self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_ALLOW_NON_ROOT_SILENT_INSTALL"] = "false"
		self.LOGGER.traceExit()
	#endDef
	
	"""
		Called by CellDefinitionReader after all topology objects have been discovered
		to gather the variables which have been extracted from the configuration
		
		These variables will be added to the env_cell.properties file.	
	"""	
	def getVariables(self):
		
		self.environmentDict["IHS_NODES"] = ",".join(self.ihsNodes)
		if (self.ihsNodeNum > 0):
			self.environmentDict["IHS_VERSION"] = self.discoveredNodes.values()[0].getNodeVersion()
			self.environmentDict["INSTALL_IHS"] = "yes"
		else:
			self.environmentDict["IHS_VERSION"] = ""
			self.environmentDict["INSTALL_IHS"] = "no"
		#endIf
		self.environmentDict["NUMBER_OF_IHS_NODES"] = str(self.ihsNodeNum)
		return self.environmentDict
	#endDef
	
#endClass

		
