'''
Licensed Materials - Property of IBM Corp. 
IBM Rational Automation Framework 
(c) Copyright IBM Corporation 2003, 2012. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 


@author: Alden Taylor
'''
from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager

from existingcell.Node import Node

from existingcell.Constants import NEWLINE

import AdminConfig
from existingcell import Constants

"""
	Provides methods to access server configuration information
"""
class Server:

	PORTAL_SERVER = "PORTAL_SERVER"
	PROXY_SERVER = "PROXY_SERVER"
	APP_SERVER = "APPLICATION_SERVER"
	APP_MEMBER = "APPLICATION_SERVER_MEMBER"
	NODE_AGENT = "NODE_AGENT"
	ADMIN_AGENT = "ADMIN_AGENT"
	DMGR_SERVER = "DEPLOYMENT_MANAGER"
	ODR_SERVER = "ONDEMAND_ROUTER"
	WEB_SERVER = "WEB_SERVER"
	
	PROP_SERVER_NAME = "SERVER_NAME"
	PROP_SERVER_TYPE = "SERVER_TYPE"
	PROP_SERVER_TEMPLATE = "SERVER_TEMPLATE"
	PROP_CLUSTER_NAME = "CLUSTER_NAME"
	
	def __init__(self, serverId, platform, isAdminAgentCell="false"):
		self.LOGGER = _Logger("Server", MessageManager.RB_WEBSPHERE_WAS)
		self.VE_LOGGER = _Logger("WveCellProcessor", MessageManager.RB_WEBSPHERE_WVE)
		
		self.serverId = serverId
		self.clusterName = ""
		self.serverType = ""
		self.serverName = ""
		self.serverNum = 0
		self.nodeNum = 0
		self.environmentDict = {}
		self.serverProperties = {}
		self.isAdminAgentCell = isAdminAgentCell
		self.platform = platform
	#endDef

	"""
		Called when a server is discovered in the node configuration
		
		@param node: The Node object representing the node that this server is on
		@param serverId: The WAS server id of the server
	"""
	def identifyServer(self, node, nodeNum, serverName=""):
		self.LOGGER.traceEnter()
		self.node = node
		self.nodeNum = nodeNum
		
		"""
			Code block for discovering server name and server type
		"""
		self.serverType = AdminConfig.showAttribute(self.serverId, 'serverType')
		if( self.isAdminAgentCell=="true" and serverName=="" ):
			self.serverType = Server.ADMIN_AGENT
		elif( self.isAdminAgentCell=="true" and serverName!="" ):
			self.serverType = Server.APP_SERVER
		elif( self.serverType == Server.APP_SERVER):
			isClusterMember = ( None != AdminConfig.showAttribute(self.serverId,'clusterName') )
			isPortal = self._isWpServer()
			if(isPortal):
				self.serverType = Server.PORTAL_SERVER
			else:
				if(isClusterMember):
					self.serverType = Server.APP_MEMBER
				else:
					self.serverType = Server.APP_SERVER
				#endif
			#endif 
		#endif
		
		# for admin managed cells, we need to set serverName earlier in the process
		# for all others, this should be true
		if( self.serverName=="" ):
			self.serverName = AdminConfig.showAttribute(self.serverId, 'name')
		#endif
		
		# set the cluster property only when relevant
		self.clusterName = self._lookupClusterName()
		if( self.clusterName != ""):
			self.serverProperties[Server.PROP_CLUSTER_NAME] = self.clusterName		
		
		## currently no way to identify the template name
		self.serverProperties[Server.PROP_SERVER_TEMPLATE] = ""
		self.serverProperties[Server.PROP_SERVER_NAME] = self.serverName
		if (self.serverType == Server.DMGR_SERVER):
			self._dmgrServerDiscovered()
		elif (self.serverType == Server.NODE_AGENT):
			self._nodeAgentServerDiscovered()
		elif (self.serverType==Server.APP_SERVER or self.serverType==Server.APP_MEMBER):
			self._appServerDiscovered()
		elif (self.serverType == Server.PORTAL_SERVER):
			self._portalServerDiscovered()
		elif (self.serverType == Server.ODR_SERVER):
			self._odrServerDiscovered()
		elif (self.serverType == Server.PROXY_SERVER):
			self._proxyServerDiscovered()
		elif (self.serverType == Server.ADMIN_AGENT):
			self._adminAgentServerDiscovered()
		else:
			self.LOGGER.log("CRWWA0187I", [self.serverType, node.getNodeName()])
		#endIf
		
		self.serverProperties[Server.PROP_SERVER_TYPE] = self.serverType
		
		self.serverNum = self.node.getNumServers()
		
		self.LOGGER.traceExit()
	#endDef	

	"""
	If this is a member of a cluster, returns the cluster name, else ""	
	"""
	def _lookupClusterName(self):
		clusterName = AdminConfig.showAttribute(self.serverId, 'clusterName')
		if (clusterName is not None):
			return clusterName.strip()
		else:
			return ""
		#endIf
	#endDef

	def getServerType(self):
		return self.serverType
	#endDef
	
	def getServerName(self):
		return self.serverName
	#endDef

	def getClusterName(self):
		return self.clusterName
	#endDef

	def getProperties(self):
		## before returning the properties, we should know by now our node and server numbers
		## add those values to the returned dictionary
		## only add properties for servers which are not DMGR or NODE_AGENT
		if (self.serverType != Server.DMGR_SERVER and self.serverType != Server.NODE_AGENT):
			for key in self.serverProperties.keys():
				prop = str(self.node.getPrefix())
				prop = prop + "_" + Constants.SERVER + str(self.serverNum)
				prop = prop + "_" + key
				self.environmentDict[prop] = self.serverProperties[key]
			#endFor
		#endIf
		return self.environmentDict
	#endDef
	
	"""	
		Retrieves the needed properties related to an application server.
		
		Properties are stored in self.environmentDict
		
		Can be overridden to set different properties on Application Server discovery
	"""	
	def _appServerDiscovered(self):
		self.node.nodeContainsAppServers = 1
		isDynamicCluster = self._isMemberOfDynamicCluster()
		if (not isDynamicCluster):
			self.LOGGER.log("CRWWA0141I", [self.node.getNodeName(), self.serverName])
			self.node.addServer(self)		
		#endIf
	#endDef

	def _proxyServerDiscovered(self):
		self.LOGGER.log("CRWWA0188I", [self.serverType, self.node.getNodeName(), self.serverName])
		self.node.addServer(self)		
	#endDef
	
	def _portalServerDiscovered(self):
		self.LOGGER.log("CRWWA0141I", self.node.getNodeName())
		self.node.nodeContainsAppServers = 1
		self.node.addServer(self)
		self.node.addWPServer(self.getServerName())
	#endDef

	"""
		True if this server represents a WP server.  This is determined by the existence
		of the WPSlib on the application server's classloader definition.
	"""
	def _isWpServer(self):
		server = self.serverId
		self.LOGGER.traceEnter(server)
		isWpServer = 0
		appServer = AdminConfig.list( "ApplicationServer", server ).split( NEWLINE )[0]
		if (len(appServer) > 0):
			for classLoader in AdminConfig.list( "Classloader", appServer ).split( NEWLINE ):
				if (len(classLoader) > 0):
					for library in AdminConfig.list( "LibraryRef", classLoader ).split( NEWLINE ):
						if (len(library) > 0):
							libraryName = AdminConfig.showAttribute(library, "libraryName")
							## WP 6.0 and 6.1 app servers will have a WPSlib defined 
							## it is required for the app server to run portal
							if (libraryName == "WPSlib"):
								isWpServer = 1
								break
							#endIf
						#endIf
					#endFor
				#endIf
			#endFor
		#endIf
		self.LOGGER.traceExit(isWpServer)
		return isWpServer
	#endDef	
	
	def _isMemberOfDynamicCluster(self):	
		## Exclude server name if it belongs to a dynamic cluster.
		## Below code takes care of that
		if(self.isAdminAgentCell == "true"):
			return 0
		
		clusterName = AdminConfig.showAttribute(self.serverId, 'clusterName')
		types = AdminConfig.types().split(NEWLINE)
		if 'DynamicCluster' in types:
			dynClusters = AdminConfig.list('DynamicCluster')
		else:
			## DynamicCluster is not a valid config type, so this is not a dynamic cluster
			self.LOGGER.trace("Server must not be a member of a dynamic cluster because the type 'DynamicCluster' is not recognized.")
			dynClusters = ""
		#endTry
		if (clusterName is not None):
			for dynamicCluster in dynClusters.split(NEWLINE):
				if(dynamicCluster == ""):
					continue
				#endIf
				dynamicClusterName = AdminConfig.showAttribute (dynamicCluster, 'name')
				if (clusterName == dynamicClusterName):
					## Server is a member of the dynamic cluster, set the flag
					return 1
				#endIf
			#endFor
		#endIf
		return 0
	#endDef

	
	"""
		Retrieves the needed properties related to a node which contains a 
		dmgr server. (A dmgr node)
		
		Properties are stored in self.environmentDict
		
		Can be overridden to set different properties on dmgr discovery
	"""
	def _dmgrServerDiscovered(self):
		self.LOGGER.traceEnter()
		node = self.node
		self.node.setDmgrServer(self)
		self.LOGGER.log("CRWWA0140I", node.getNodeName())
		nodeVars = node.getNodeVariables()
		self.environmentDict["ND_HOST_NAME"] = AdminConfig.showAttribute(node.getConfigId(), "hostName")
		self.environmentDict["ND_NODE_NAME"] = AdminConfig.showAttribute(node.getConfigId(), "name")
		portsByServer = node.getPorts()
		## pick the first ports definition -- it'll be the dmgr server
		portsDict = portsByServer.values()[0]
		if (portsDict.has_key("SOAP_CONNECTOR_ADDRESS")):
			self.environmentDict["ND_SOAP_PORT"] = portsDict["SOAP_CONNECTOR_ADDRESS"]
		else:
			self.environmentDict["ND_SOAP_PORT"] = "8879"
		#endIf
		startingPoint = self.computeLowOrderPort(portsDict)
		self.environmentDict["ND_STARTING_POINT"] = startingPoint
		if (nodeVars.has_key("WAS_INSTALL_ROOT")):
			self.environmentDict["ND_WAS_HOME"] = nodeVars['WAS_INSTALL_ROOT']
		else:
			raise "VariableSubstitutionEntry with name='WAS_INSTALL_ROOT' is not defined in the WebSphere configuration."
		#endIf
		if (nodeVars.has_key("USER_INSTALL_ROOT")):
			self.environmentDict["ND_WAS_PROF_HOME"] = nodeVars['USER_INSTALL_ROOT']
		else:
			self.environmentDict["ND_WAS_PROF_HOME"] = self.environmentDict["ND_WAS_HOME"] + "/profiles/" + node.getProfileName()
		#endIf
		self.environmentDict["ND_PROFILE_NAME"] = node.getProfileName()
		if (self.platform == "zos"):
			self.environmentDict["ND_OS"] = "ZOS"
		else:
			self.environmentDict["ND_OS"] = "Non-ZOS"
		#endIf
		self.LOGGER.traceExit()
	#endDef

	"""	
		Retrieves the needed properties related to a node agent server.
		
		Properties are stored in self.environmentDict
		
		Can be overridden to set different properties on Node Agent discovery
	"""	
	def _nodeAgentServerDiscovered(self):
		self.LOGGER.log("CRWWA0142I", self.node.getNodeName())
		self.node.addNodeAgentServer(self)
		node = self.node
		node.nodeContainsAppServers = 1
		portsByServer = node.getPorts()
		nodeAgentStartingPort = ""
		if (portsByServer.has_key(self.serverName)):
			nodeAgentStartingPort = self.computeLowOrderPort(portsByServer[self.serverName])
		#endIf
			
		## increase the number of nodes in the topology
		## this will only be incremented for ND for each node
		self.nodeNum += 1
		node.managedNodeNum = self.nodeNum
		self.environmentDict[Constants.NODE + str(self.nodeNum) + "_NODEAGENT_STARTING_POINT"] = nodeAgentStartingPort
	#endDef	

	"""    
		Retrieves the needed properties related to an ODR server.
		Properties are stored in self.environmentDict
	"""
	def _odrServerDiscovered(self):
		self.LOGGER.traceEnter()
		self.node.nodeContainsAppServers = 1
		self.VE_LOGGER.log("CRWVE0152I", self.node.getNodeName())
		self.node.addServer(self)
		self.LOGGER.traceExit()
	#endDef

	"""    
		Retrieves the needed properties related to an admin agent server.
		Properties are stored in self.environmentDict
	"""
	def _adminAgentServerDiscovered(self):
		self.LOGGER.traceEnter()
		self.node.nodeContainsAppServers = 1
		self.LOGGER.log("CRWWA0189I", self.node.getNodeName())
		self.serverType = Server.ADMIN_AGENT
		self.node.addServer(self)
		self.LOGGER.traceExit()
	#endDef
	
	"""
		ports is a dict of portName = portValue pairs
		if the port values are a sequence, returns the first port
		if the port values are not in a sequence, returns ""
	"""
	def computeLowOrderPort(self, ports):
		portVals = ports.values();
		portValInt = []
		for val in portVals:
			portValInt.append(int(val))
		#endFor
		portValInt.sort();
		if (len(portValInt) > 1):
			first = portValInt[0]
			diff = len(portValInt) - 1
			last = portValInt[diff]
			if (first + diff == last):
				# numbers are in sequence
				return first
			#endIf
		#endIf
		return ""
	#endDef

	## Finds the name of the configuration object using the configId 
	def findName(self, configId):
		name = None
		if (configId.find('"') >=0):
			# Eg: '"My Bus(cells/cell/buses/My Bus|sib-bus.xml)"'
			name = configId.split('"')[1].split('(')[0]
		else:
			# Eg: 'Bus1(cells/cell/buses/Bus1|sib-bus.xml)'
			name = configId.split('(')[0]
		#endIf
		
		return name
	#endDef

#endClass
