'''
Licensed Materials - Property of IBM Corp. 
IBM Rational Automation Framework 
(c) Copyright IBM Corporation 2003, 2012. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 


@author: Vandana Shenoy
'''
from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager
from com.ibm.rational.rafw.wsadmin.util import WsadminStringUtils
from java.util import ArrayList
from java.io import File
from existingcell.Node import Node

from existingcell.Constants import DR_DEFAULT_VALUE
from existingcell.Constants import NEWLINE
from existingcell.AbstractCellProcessor import AbstractCellProcessor
from existingcell.Server import Server


import AdminConfig
import AdminTask
import AdminControl

"""
    Processor that gathers WVE topology information from a runtime configuration
"""
class WveCellProcessor(AbstractCellProcessor):
	
	def __init__(self, cellDefinitionReader):
		AbstractCellProcessor.__init__(self, cellDefinitionReader)
		self.LOGGER_WAS = _Logger("WveCellProcessor.WAS", MessageManager.RB_WEBSPHERE_WAS)
		self.LOGGER = _Logger("WveCellProcessor", MessageManager.RB_WEBSPHERE_WVE)
		self.environmentDict = {}
		self.clusterNum = 0
		self.clusters=""
		self.cellType="WVE"
		self.wveVersion = self.getWveVersion()
		## read node data
		self.managedNodeNum = 0
		self.nodes = []
		self.discoveredNodes = {}
		self.discoveredServers = {}
		## List of dynclusters
		self.dynclusters = ""
		self.platform = ""
	#endDef
	
	"""
		Retrieves the needed properties related to a node which contains an 
		application server.
		
		Properties are stored in self.environmentDict
	"""
	def _processAppServerNode(self, node):
		self.LOGGER.traceEnter()
		nodeVars = node.getNodeVariables()
		if (int(self.wveVersion) > 80 and self.isStandalone):
			self.environmentDict["BASE_STARTING_POINT"] = ""
		else:
			self._setNodeProperty("_BASE_STARTING_POINT", "")
		self._setNodeProperty("_HOST_NAME", AdminConfig.showAttribute(node.getConfigId(), "hostName"))
		nodeName = node.getNodeName()
		self._setNodeProperty("_NODE_NAME", nodeName)
		self._setNodeProperty("_PROFILE_NAME", node.getProfileName())
		
		if (nodeVars.has_key("WAS_INSTALL_ROOT")):
			self._setNodeProperty("_WAS_HOME", nodeVars['WAS_INSTALL_ROOT'])
		else:
			raise "VariableSubstitutionEntry with name='WAS_INSTALL_ROOT' is not defined in the WebSphere configuration."
		#endIf
		if (nodeVars.has_key("USER_INSTALL_ROOT")):
			self._setNodeProperty("_WAS_PROF_HOME", nodeVars['USER_INSTALL_ROOT'])
		else:
			self.environmentDict["ND_WAS_PROF_HOME"] = nodeVars['WAS_INSTALL_ROOT'] + "/profiles/" + node.getProfileName()
		#endIf
		
		self._setNodeProperty("_NODE_TYPE", node.getDataType())
		
		if (int(self.wveVersion) > 80 and self.isStandalone):
			portsByServer = node.getPorts()
			if (portsByServer.has_key("server1")):
				## user server1's SOAP port
				ports = portsByServer["server1"]
			else:
				## pick the first ports definition
				ports = portsByServer.values()[0]
			#endIf
			if (ports.has_key("SOAP_CONNECTOR_ADDRESS")):
				self._setNodeProperty("_SOAP_PORT", ports["SOAP_CONNECTOR_ADDRESS"])
			else:
				self._setNodeProperty("_SOAP_PORT", "8880")
			#endIf
		#endIf
		if (self.platform == "zos"):
			self._setNodeProperty("_OS", "ZOS")
		else:
			self._setNodeProperty("_OS", "Non-ZOS")
		#endIf
		self.LOGGER.traceExit()
	#endDef
	
	"""
		Sets a property in the environment dictionary, also used by sub classes
	"""
	def _setNodeProperty(self, keySuffix, value):
		self.LOGGER.traceEnter()
		
		if (int(self.wveVersion) > 80 and self.isStandalone):
			self.environmentDict["BASE" + keySuffix] = value
		else:
			self.environmentDict["NODE" + str(self.managedNodeNum) + keySuffix] = value
		#endIf
		
		self.LOGGER.traceExit()
	#endDef
	
	"""
		Called by CellDefinitionReader when a cell is discovered. 
	"""
	def cellDiscovered(self, cellId):
		self.LOGGER.traceEnter()
		# The very fact that we have detected a WVE cell and created an instance of WveCellProcessor, means 
		# that the CELL_TYPE HAS to be "WVE". Also we only support the DISTRIBUTED cell type for WVE,
		# hence, that is also implicit.
		if (int(self.wveVersion) > 80):
			self.LOGGER_WAS.log("CRWWA0137I", "WAS")
			self.cellType = "WAS"
		else:
			self.LOGGER_WAS.log("CRWWA0137I", "WVE")
		#endIf
		
		cellType=AdminConfig.showAttribute(cellId,'cellType')
		
		if (cellType == "STANDALONE"):
			self.isStandalone = 1
		elif (cellType == "DISTRIBUTED"):
			self.isStandalone = 0
		else:
			self.LOGGER.log("CRWWA0138I")
			self.isStandalone = 0
		#endIf
		self.LOGGER.traceExit()
	#endDef
	
	"""
		Called by CellDefinitionReader when a node is discovered in the cell configuration
	"""
	def nodeDiscovered(self, nodeId):
		self.LOGGER.traceEnter()
		nodeConfig = Node(nodeId)
		self.discoveredNodes[nodeId] = nodeConfig
		self.LOGGER_WAS.log("CRWWA0139I", nodeConfig.getNodeName())
		self.LOGGER.traceExit()
	#endDef
	
	"""
		Called by CellDefinitionReader when a server is discovered in the node configuration
	"""
	def serverDiscovered(self, nodeId, serverId):
		self.LOGGER.traceEnter()
		node = self.discoveredNodes[nodeId]
		server = Server(serverId, self.platform)
		server.identifyServer(node, self.managedNodeNum)

		# figure out the type of node based on the server
		nodeType = self.getNodeTypeFromServer(server)
		node.setDataType(nodeType)
		if (nodeType == Node.MANAGED_NODE):
			self.managedNodeNum += 1
			self.nodes.append(node.getNodeName())
		#endIf
		
		self.discoveredServers[server.getServerName()] = server
		self.LOGGER.traceExit()
	#endDef
	
	"""
		Returns the WVE cell version as a 2 character string.  [major,minor]
		Example: 61 / 70
	"""
	def getWveVersion(self):
		self.LOGGER.traceEnter()
		dmgrNode = AdminControl.getNode()
		productVersion = AdminTask.getMetadataProperty ('[-nodeName ' + dmgrNode + ' -propertyName com.ibm.websphere.wxdopProductVersion]')
		major,minor,trash = productVersion.split('.', 2)
		version = major + minor
		self.LOGGER.traceExit(version)
		return version
	#endDef
	
	"""
        Called by CellDefinitionReader when a cluster is discovered in the cell configuration
    """    
	def clusterDiscovered(self, clusterId):
		self.LOGGER.traceEnter()
		self.clusterNum +=1
		clusterType = ""
		dynamicClusterName = ""
		clusterName = AdminConfig.showAttribute(clusterId, 'name')
		clusterWasType = AdminConfig.showAttribute(clusterId,'serverType')
		
		self.clusters = self.clusters + clusterName + ","
		
		## In the future, we could copy this if statement into the loop to support multiple
		## ODR clusters, but for now, we only support one so the loop has to stay in the else case
		if (clusterWasType == "ONDEMAND_ROUTER"):
			clusterType = "ODR"
			self.environmentDict["ODR_CLUSTER_NAME"] = clusterName;
			self.environmentDict["SERVER_TEMPLATE"] = DR_DEFAULT_VALUE
			self.environmentDict["ODR_TEMPLATE"] = DR_DEFAULT_VALUE
		else:
			## If there are dynamic clusters, iterate through those, and set the clusterType and add to
			## the DYNCLUSTERS list
			for dynamicCluster in AdminConfig.list('DynamicCluster').split(NEWLINE):
				if(dynamicCluster == ""):
					continue
				#endIf
				dynamicClusterName = AdminConfig.showAttribute (dynamicCluster, 'name')
				## If the names match, then the cluster we are processing is a Dynamic Cluster
				if (dynamicClusterName == clusterName):
					self.dynclusters = self.dynclusters + clusterName + ","
					self.environmentDict["CLUSTER" + str(self.clusterNum) + "_SERVER_TYPE"] = "WAS"
					self.environmentDict["CLUSTER" + str(self.clusterNum) + "_MEMBERSHIP_POLICY"] = ""
					clusterType = "DYNAMIC_WAS"
					break;
				#endIf
			#endFor
			if(None==clusterType or ""==clusterType):
				clusterType = "WAS"
		#endIf
		
		## Printing the type of cluster discovered :
		if (clusterType == "WAS"):
			self.LOGGER_WAS.log("CRWWA0143I", clusterName)
		elif (clusterType == "ODR"):
			self.LOGGER.log("CRWVE0153I", clusterName)
		else:
			self.LOGGER.log("CRWVE0154I", clusterName)
		#endIf
		self.environmentDict["CLUSTER" + str(self.clusterNum)] = clusterName
		self.environmentDict["CLUSTER" + str(self.clusterNum) + "_CLUSTER_NAME"] = clusterName
		self.environmentDict["CLUSTER" + str(self.clusterNum) + "_TRANSPORT_NODE_INCREMENTOR"] = "0"
		self.environmentDict["CLUSTER" + str(self.clusterNum) + "_TRANSPORT_STARTING_POINT"] = "0"
		serverNodeMapping = {}
		
		serverNames = ArrayList()
		for clusterMember in AdminConfig.list('ClusterMember', clusterId).split(NEWLINE):
			if (len(clusterMember) > 0):
				serverName = AdminConfig.showAttribute(clusterMember, 'memberName')
				serverNames.add(serverName)
				nodeName = AdminConfig.showAttribute(clusterMember, 'nodeName')
				self.LOGGER_WAS.log("CRWWA0144I", [serverName, nodeName])
				if (not serverNodeMapping.has_key(nodeName)):
					serverNodeMapping[nodeName] = ArrayList()
				#endIf
				serverNodeMapping[nodeName].add(serverName)
			#endIf
		#endFor
		prefix = WsadminStringUtils.findCommonPrefix(serverNames)
		if (len(prefix) > 0):
			self.environmentDict["CLUSTER" + str(self.clusterNum) + "_PREFIX"] = prefix
		else:
			self.environmentDict["CLUSTER" + str(self.clusterNum) + "_PREFIX"] = clusterName + "_clone"
		#endIf
		nodes = ""
		maxPerNode=0
		for nodeName in serverNodeMapping.keys():
			nodes = nodes + nodeName + ","
			if (serverNodeMapping[nodeName].size() > maxPerNode):
				maxPerNode = serverNodeMapping[nodeName].size()
			#endIf
		#endFor
		## For a dynamic cluster, we do not want to populate the property CLUSTER_{NUM}_NODES
		if (clusterType == "DYNAMIC_WAS"):
			self.environmentDict["CLUSTER" + str(self.clusterNum) + "_NODES"] = ""
		else:
			self.environmentDict["CLUSTER" + str(self.clusterNum) + "_NODES"] = (nodes[0:len(nodes)-1])
		#endIf
		self.environmentDict["CLUSTER" + str(self.clusterNum) + "_PERNODE"] = str(maxPerNode)		
        
		self.environmentDict["CLUSTER" + str(self.clusterNum) + "_CLUSTER_TYPE"] = clusterType
		self.LOGGER.traceExit()
	#endDef
	
	def _processAdditionalProperties(self):
		self.LOGGER.traceEnter()
		if (int(self.wveVersion) > 80):
			self.environmentDict["CELL_TYPE"] = "WAS"
			self.environmentDict["PRODUCT_VERSION"] = "WAS" + self.wveVersion
			self.environmentDict["WAS_VERSION"] = self.wveVersion
		else:
			self.environmentDict["CELL_TYPE"] = self.cellType
			self.environmentDict["PRODUCT_VERSION"] = "WVE" + self.wveVersion
			self.environmentDict["WVE_VERSION"] = self.wveVersion
		#endIf
		self.environmentDict["PRIMARY_NODE"] = ""
		self.environmentDict["ENABLE_SECURITY_AT_CREATION"] = "true"
		self.environmentDict["USING_DB2"] = "false"
		self.environmentDict["DB2_CLIENT_HOME"] = ""
		self.environmentDict["ALLOW_NON_ROOT_SILENT_INSTALL"] = "true"
		self.LOGGER.traceExit()
	#endDef
	
	"""
		Called by CellDefinitionReader after all topology objects have been discovered
		to gather the variables which have been extracted from the configuration
		
		These variables will be added to the env_cell.properties file.    
	"""
	def getVariables(self):
		self.LOGGER.traceEnter()
		numberOfNodes = self.managedNodeNum
		wasVersion = ""
		
		## wasFullVersion is in the form 7.0.0.39
		wasFullVersion = ""
		for node in self.discoveredNodes.values():
			if (node.nodeContainsAppServers):
				self.managedNodeNum = node.managedNodeNum
				self._processAppServerNode(node)
				self._setNodeProperty("_SERVERS_ON_NODE", node.getServersOnNodeStr())
				self._setNodeProperty("_STANDALONE_SERVERS_ON_NODE", node.getStandaloneServersOnNodeStr())
				self.environmentDict["LOAD_BALANCE_SERVERS_ON_NODE" ] = node.getStandaloneServersOnNodeStr()
				self._setNodeProperty("_NUMBER_OF_SERVERS", str(node.getNumServers()))
				self._setNodeProperty("_NUMBER_OF_STANDALONE_SERVERS", str(node.getNumStandaloneServers()))
			#endIf
			## add all server properties
			node.setServerProperties(self.environmentDict)
			
			## all nodes should have the same version
			wasVersion = node.getNodeVersion()
			nodeFullVersion = node.getNodeFullVersion()
			self._setNodeProperty("_WAS_FULL_VERSION", str(nodeFullVersion))
			## take the largest version number in the cell 
			## (this will fail for things like 7.0.0.9 vs 7.0.0.19, but it rarely will happen)
			if (nodeFullVersion > wasFullVersion):
				wasFullVersion = nodeFullVersion
		#endFor
		self.environmentDict["WAS_VERSION"] = wasVersion
		self.environmentDict["WAS_FULL_VERSION"] = str(wasFullVersion)
		
		if (int(self.wveVersion) > 80 and self.isStandalone):
			self.environmentDict["NODES"] = ""
			self.environmentDict["LOAD_BALANCE_NODES"] = ""
			self.environmentDict["NUMBER_OF_NODES"] = str(0)
			self.environmentDict["IS_BASE"] = "yes"
		else:
			self.environmentDict["NODES"] = ",".join(self.nodes)
			self.environmentDict["LOAD_BALANCE_NODES"] = ",".join(self.nodes)
			self.environmentDict["NUMBER_OF_NODES"] = numberOfNodes
			self.environmentDict["IS_BASE"] = "no"
		#endIf
		
		##Update the environment Dictionary for DYNCLUSTERS
		if (len(self.dynclusters) > 0):    
			self.environmentDict["DYNCLUSTERS"] = (self.dynclusters[0:len(self.dynclusters)-1])
		else:
			self.environmentDict["DYNCLUSTERS"] = ""
		#endFor
		
		if (len(self.clusters) > 0):    
			self.environmentDict["CLUSTERS"] = (self.clusters[0:len(self.clusters)-1])
		else:
			self.environmentDict["CLUSTERS"] = ""
		#endIf
		self.environmentDict["NUMBER_OF_CLUSTERS"] = str(self.clusterNum)
		self._processAdditionalProperties()
		self.LOGGER.traceExit()
		return self.environmentDict
	#endDef
#endClass